<?php
namespace Teknisa\CMP\Modules\Fornecimento\Modules\AutorizacaoFornecimento\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\ORM\DateTime as DateTimeZeedhi;
use Teknisa\Libs\Util\DateUtil;

class Sup05100AnaliseAFController extends \Teknisa\GER\Controllers\TeknisaCrud {
    const TIPO = 4;
    const IDTIPOLANITM = 'M';
    const EXCLUSAO = 'Exclusão do Item.';
    const EXCLUSAO_ANALISE = 'Análise de AF-Exclusão do item';
    const EXCLUSAO_ANALISE_REM = 'Análise de AF-Excl. do item/Remanej.';
    const INCLUSAO_ANALISE = 'Análise de AF-Inclusão do Item';
    const INCLUSAO_ANALISE_REM = 'Análise de AF-Incl. do Item/Remanej.';
    const ALTER_DTENT_AF = 'Alteração Dt. Entrega de: ';
    const ALTER_DTENT_ANALISE = 'Análise de AF-Alteração na Dt. Entrega';
    const ALTER_QTDE_AF = 'Alteração da quantidade de: ';
    const ALTER_QTDE_ANALISE = 'Análise de AF-Alteração na Quantidade';
    const ALTER_QTDE_ANALISE_REM = 'Análise de AF-Alter.da Qt./Remanej.';
    const PARA = ' para ';
    const DEFAULT_EMAIL_TEMPLATE = "Prezado Fornecedor, <i>[Nome Fornecedor]</i><br/><br/>Parabéns! Você foi selecionado em nosso processo de cotação.<br/>A Autorização de Fornecimento foi disponibilizada em nosso Portal de Compras.<br/><br/>Acesse <i>[link do portal]</i> e confira no Número de Lote <i>[nº do lote]</i> das AFs disponíveis, ou consulte o relatório em anexo.<br/><br/>Lembramos que não são permitidas entregas em desacordo com a Autorização de Fornecimento. Sendo o fornecedor responsável por divergências ou possíveis devoluções.<br/><br/>Verifique atentamente o local de entrega, dados de faturamento de cada unidade, produtos, marcas, preços, embalagens, quantidades e demais informações.<br/><br/>Siga rigorosamente a Autorização de Fornecimento!<br/><br/>Att<br/><br/>";
    const EMAIL_COOKIE_NAME = 'MAIL_BODY_CONCLUIR_ANALISE';
    const RESEND_EMAIL_COOKIE_NAME = 'MAIL_BODY_RESEND';
    const SUBJECT_EMAIL_COOKIE_NAME = 'MAIL_SUBJECT_CONCLUIR_ANALISE';
    const SUBJECT_RESEND_EMAIL_COOKIE_NAME = 'MAIL_SUBJECT_RESEND';
    const SEND_TO_OPERATOR = 'SEND_TO_OPERATOR';
    const RESEND_TO_OPERATOR = 'RESEND_TO_OPERATOR';
    const MENSAGENS = array(
        'FORN_N_ATENDE_PERIODICIDADE' => "Fornecedor não atende a data de entrega informada devido à sua periodicidade.",
        'AF_EXISTENTE_NA_DATA'        => "Operação bloqueada! <br> Favor realizar o remanejamento de AF, pois já existe uma AF para esse item na data informada.",
        'N_Ha_AF_Cons'                => "Operação bloqueada. Todas as AF's da apuração estão inconsistentes e/ou fora do limite.", //'AFS_INCONS_FORA_LIMITE'      => "Operação bloqueada! As AF's estão inconsistentes e/ou fora do limite.",
        'ANALISE_AF_CONCLUIDA'        => "Operação bloqueada. Esta análise de autorização de fornecimento já foi concluída.",
        'Compr_NCAD'                  => "",
        'Comprad_Inf'                 => "",
        'Erro_LibAF'                  => "Operação bloqueada. Uma ou mais AF's ultrapassaram o limite de liberação do gerente.",
        'GERENTE_COMP_N_CAD'          => "Não há gerente cadastrado para comprador.",
        'SENHA_INVALIDA'              => "Senha inválida",
        'Oper_Relac_N_Aut'            => "Operador não autorizado a realizar alterações sobre AF's desde comprador.",
        'Oper_N_Relac'                => "Operador não relacionado ao comprador.",
        'Linha_LogAF_AGRUP_OK'        => "Foram enviadas AF's para %s pelo sistema.",
        'Linha_LogAF_AGRUP_ERRO'      => "Não foram enviadas AF's para %s pelo sistema.",
        'Linha_LogAF_PORTAL_OK'       => "Foram enviadas AF's para %s através do portal.",
        'Linha_LogAF_PORTAL_ERRO'     => "Não foram enviadas AF's para %s através do portal.",
        'Email_S_AF_ERRO'             => "Porém o aviso via e-mail não foi enviado.",
        'FORN_SEM_EMAIL'              => "Fornecedor não possui e-mail cadastrado.",
        'NAO_HA_DADOS'                => "Não há dados a ser listado.",
        'MAIL_NOT_SENDED_IDTPENVAFFOR'=> "Fornecedor parametrizado para não receber email.",
        'Linha_LogAF_REENVIO_ERRO'    => "Não há informação para envio de e-mail de acordo com o filtro utilizado."
    );
    
    protected $crudService;
    protected $parameterBag;
    protected $dataSourceName;
    protected $projectCorp;
    protected $sup05100AnaliseAFService;
    protected $fechamentoAF;
    protected $contaPagar;
    protected $newCode;
    protected $reportController;
    protected $logEnvioAF;
    
    public function __construct($crudService, $parameterBag, $projectCorp, $sup05100AnaliseAFService, $fechamentoAF, $contaPagar, $environment, $newCode, $reportController, $cmpUtilities) {
        $this->crudService              = $crudService;
        $this->parameterBag             = $parameterBag;
        $this->projectCorp              = $projectCorp;
        $this->sup05100AnaliseAFService = $sup05100AnaliseAFService;
        $this->fechamentoAF             = $fechamentoAF;
        $this->contaPagar               = $contaPagar;
        $this->environment              = $environment;
        $this->newCode                  = $newCode;
        $this->reportController         = $reportController;
        $this->cmpUtilities             = $cmpUtilities;
        $this->logEnvioAF               = array( 'REPORT' => array(), 'SUCESSO' => array(), 'ERRO' => array(), 'AF_INCONSIST' => false, 'NEW_NRAPURPREC' => null );
    }
    
    
    /********************************************
     *               Utilitários                *
     ********************************************/
     
    public function generateCode($tableName) {
        return $this->newCode->getCodeByTableName($tableName, $this->environment->getNrOrg());
    }
    
    public function retornaValorParamFilterCriteria($filterCriteria, $nomeParam) {
        $conditions = $filterCriteria->getConditions();
        
        foreach($conditions as $condition){
            if($condition['columnName'] == $nomeParam) {
                return $condition['value'];
            }
        }
    }
    
    public function stringParaFloat($valor) {
        if(gettype($valor) === 'string') {
            $valor = str_replace('.', '', $valor);
            $valor = str_replace(',', '.', $valor);
            
            $valor = floatval($valor);
        }
        
        return $valor;
    }
    
    public function formataDataString($data) {
        if(gettype($data) === 'string') {
            $data = substr($data, 0, 10);
            $data = str_replace('/', '-', $data);
            $data = new \DateTime($data);
            $data = $data->format('d/m/Y');
        } else if($data instanceof \DateTime) {
            $data = $data->format('d/m/Y');
        }
        
        return $data;
    }
    
    public function prox_Lista_AF($stListaAF) {
        $stNrLista = '';
        
        if ($stListaAF == 'T') {
            $stNrLista = self::generateCode('LISTA');
        } else {
            if (!empty($stListaAF)) {
                $stNrLista = self::generateCode('LISTA');
                foreach ($stListaAF as $af) {
                    $paramsAcmInsLista = array(
                        'NRLISTA' => $stNrLista,
                        'CDLISTA' => $af // $stAuxListaAF
                    );
                    $this->sup05100AnaliseAFService->acmInsLista($paramsAcmInsLista);
                }
            }
        }
        
        return $stNrLista;
    }
    
    public function delete_Lista($stLista) {
        $this->sup05100AnaliseAFService->acmDelLista(array( 'P_NRLISTA' => $stLista ));
    }
    
    public function addMessage(&$response, $mensagem) {
        if (!empty($response)) {
            $response->addMessage(new Message($mensagem));
        }
    }
    
    
    /********************************************
     *               DataSources                *
     ********************************************/
    
    public function retornaProdutosItemcole(DTO\Request\Filter $request, DTO\Response $response) {
        $filterCriteria = $request->getFilterCriteria();
        
        $particularidades = self::retornaValorParamFilterCriteria($filterCriteria, 'CONSPARTFILDEST');
        if($particularidades == 'S') {
            $this->dataSourceName = 'sup05100_produtos_itemcole_particularidades';
        } else {
            $this->dataSourceName = 'sup05100_produtos_itemcole';
        }
        
        parent::find($request, $response);
    }
    
    public function retornaApuracao(DTO\Request\Filter $request, DTO\Response $response) 
    {
        if($this->projectCorp) {
            $this->dataSourceName = 'sup05100_apuracao_corp';
        } else {
            $this->dataSourceName = 'sup05100_apuracao_oper';
        }
        
        parent::find($request, $response);
    }
    
    public function retornaProdutos(DTO\Request\Filter $request, DTO\Response $response) 
    {
        $filterCriteria = $request->getFilterCriteria();
        
        $cdfildest = self::retornaValorParamFilterCriteria($filterCriteria, 'P_CDFILDEST');
        
        $idfuncaofili = $this->sup05100AnaliseAFService->retornaFilialOL($cdfildest);
        
        if($idfuncaofili == 'O') {
            $this->dataSourceName = 'sup05100_produtos_OL';
        } else {
            $this->dataSourceName = 'sup05100_produtos';
        }
        
        parent::find($request, $response);
    }
    
    /********************************************
     *              Editar Produto              *
     ********************************************/
    
    public function acionaSalvarEdicaoProduto(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            
            if (self::isApuracaoEmAnalise($row['IDSITAPUPREC'])) {
                $result = $this->salvaEdicaoProduto($row);
            } else {
                $result = ['AF_DELETADA' => false, 'ITEM_DELETADO' => false, 'EDITED_ITEM' => false];
            }
            
            $response->addMessage(new Message($result));
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function salvaEdicaoProduto($prodEdit) {
        // $prodEdit = $request->getRow();
        $deletaAF = false;
        $deletaItemAF = false;
        $editedItemAF = false;
        
        $prodEdit['DTPROGENTAF'] = self::formataDataString($prodEdit['DTPROGENTAF']);
        $prodEdit['DTPROGENTAF_OLD'] = self::formataDataString($prodEdit['DTPROGENTAF_OLD']);
        $prodEdit['QTPROGENTAF'] = self::stringParaFloat($prodEdit['QTPROGENTAF']);
        $prodEdit['QTPROGENTAF_OLD'] = self::stringParaFloat($prodEdit['QTPROGENTAF_OLD']);
        
        try {
            $this->sup05100AnaliseAFService->iniciaConexao();
            
            if($prodEdit['QTPROGENTAF'] > 0) {
                $produto = array($prodEdit);
                
                self::verificaPeriodicidade($prodEdit['CDFILDEST'], $prodEdit['CDFORNECED'], self::formataDataString($prodEdit['DTPROGENTAF']));

                $itemautf = $this->sup05100AnaliseAFService->retornaListaItemAutfAF($prodEdit['CDFILDEST'], $prodEdit['CDFILLANC'], $prodEdit['NRAUTOFORN']);
                
                self::verificaGeraAfDataEntrega($prodEdit, $itemautf, $produto);

                self::verificaExisteAf($produto[0]['CDFILDEST'], $produto[0]['CDFILLANC'], $produto[0]['NRAPURPREC'], $produto[0]['CDPRODUTO'], $produto[0]['DTPROGENTAF'], $produto[0]['IDGERAFDTENT']);

                foreach($produto as $prod) {
                    $existeItemAF = self::retornaItemautf($itemautf, $prod);

                    if(is_object($existeItemAF)) {
                        if($prod['DTPROGENTAF_OLD'] <> self::formataDataString($existeItemAF->getDtprogentaf())) {
                            $prod['QTPROGENTAF'] = $prod['QTPROGENTAF'] + $existeItemAF->getQtprogentaf();

                            $this->sup05100AnaliseAFService->deletaItemAutf($prod['CDFILDEST'], $prod['CDFILLANC'], $prod['NRAUTOFORN'], $prod['CDPRODUTO'], $prod['DTPROGENTAF_OLD'], $prod['CDMARCA']);
                        }
                        
                        $editedItemAF = $this->sup05100AnaliseAFService->atualizaQtItemAutf($existeItemAF, $prod['QTPROGENTAF'], self::IDTIPOLANITM); 
                        
                        self::insereMovopaf($prod, 'A');

                    } else {
                        $editedItemAF = $this->sup05100AnaliseAFService->insereItemAutf($prod['DTPROGENTAF_OLD'], $prod['DTPROGENTAF'], $prod['QTPROGENTAF'], self::IDTIPOLANITM, $prod['CDFILDEST'], $prod['CDFILLANC'], $prod['NRAUTOFORN'], $prod['CDPRODUTO'], $prod['CDMARCA']);

                        $this->sup05100AnaliseAFService->deletaItemAutf($prod['CDFILDEST'], $prod['CDFILLANC'], $prod['NRAUTOFORN'], $prod['CDPRODUTO'], $prod['DTPROGENTAF_OLD'], $prod['CDMARCA']);
                        
                        self::insereMovopaf($prod, 'A');
                    }
                }
            } else {
                $deletaItemAF = $this->sup05100AnaliseAFService->deletaItemAutf($prodEdit['CDFILDEST'], $prodEdit['CDFILLANC'], $prodEdit['NRAUTOFORN'], $prodEdit['CDPRODUTO'], $prodEdit['DTPROGENTAF_OLD'], $prodEdit['CDMARCA']);
                
                $deletaAF = $this->sup05100AnaliseAFService->deletaAutoforne($prodEdit['CDFILDEST'], $prodEdit['CDFILLANC'], $prodEdit['NRAUTOFORN']);
                
                self::insereMovopaf($prodEdit, 'E', $prodEdit['DTPROGENTAF_OLD'], self::EXCLUSAO, self::EXCLUSAO_ANALISE);
            }
            
            $cdfilcentcmp = $this->sup05100AnaliseAFService->retornaCentralCompras($prodEdit['CDFILDEST']);
            $this->fechamentoAF->fechaAF($prodEdit['CDFILDEST'], $prodEdit['CDFILLANC'], $prodEdit['NRAUTOFORN'], $cdfilcentcmp);
            
            $this->updateItemSoli($prodEdit);
            
            $this->sup05100AnaliseAFService->realizaFlush();
            $this->sup05100AnaliseAFService->realizaCommit();
            
            return ['AF_DELETADA' => $deletaAF, 'ITEM_DELETADO' => $deletaItemAF, 'EDITED_ITEM' => $editedItemAF];
            
        } catch(\Exception $e) {
            $this->sup05100AnaliseAFService->realizaRollback();
            
            throw $e;
        }
        
    }
    
    public function verificaPeriodicidade($cdfilial, $cdforneced, $dtprogentaf) {
        $idUtilizaCalendario = $this->sup05100AnaliseAFService->retornaIdUtilizaCalendario($cdforneced, $cdfilial);
        $mensagem = self::MENSAGENS['FORN_N_ATENDE_PERIODICIDADE']; // 'Fornecedor não atende a data de entrega informada devido à sua periodicidade.';

        if($idUtilizaCalendario <> 'S') {
            $novaDtprogentaf = $this->sup05100AnaliseAFService->proceCarregaPeriforn($cdfilial, $cdforneced, $dtprogentaf);
        } else {
            $novaDtprogentaf = $dtprogentaf;
        }
        
        if(self::formataDataString($novaDtprogentaf) <> self::formataDataString($dtprogentaf)) {
            throw new \Exception($mensagem);
        }      
    }
    
    public function verificaGeraAfDataEntrega($prodEdit, $itemautf, &$produto) {
        if($prodEdit['ALTERADTENTREGA']) {
            foreach($itemautf as $item) {
                $itemAF = self::itemautfParaArray($item, $prodEdit['DTPROGENTAF'], $item->getQtprogentaf());
                
                $itemAF['DTPROGENTAF'] = self::formataDataString($itemAF['DTPROGENTAF']);
                $itemAF['DTPROGENTAF_OLD'] = self::formataDataString($item->getDtprogentaf());
                $itemAF['QTPROGENTAF'] = self::stringParaFloat($itemAF['QTPROGENTAF']);
                $itemAF['QTPROGENTAF_OLD'] = self::stringParaFloat($itemAF['QTPROGENTAF']);
                $itemAF['ATUSOLICITACMP'] = $prodEdit['ATUSOLICITACMP'];
                
                if($itemAF['DTPROGENTAF_OLD'] <> $itemAF['DTPROGENTAF']) {
                    if($prodEdit['CDPRODUTO'] <> $itemAF['CDPRODUTO'] || $prodEdit['CDMARCA'] <> $itemAF['CDMARCA']) {
                        self::verificaExisteAf($itemAF['CDFILDEST'], $itemAF['CDFILLANC'], $prodEdit['NRAPURPREC'], $itemAF['CDPRODUTO'], $itemAF['DTPROGENTAF'], $prodEdit['IDGERAFDTENT']);
                        array_push($produto, $itemAF);
                    }
                }
            }
        }
    }
    
    public function itemautfParaArray($itemautf, $dtprogentaf_NOVA, $qtprogentaf_NOVA) {
        $params = array(
            'CDFILDEST' => $itemautf->getCdfildest(),
            'CDFILLANC' => $itemautf->getCdfillanc(),
            'NRAUTOFORN' => $itemautf->getNrautoforn(),
            'CDMARCA' => $itemautf->getCdmarca(),
            'CDPRODUTO' => $itemautf->getCdproduto(),
            'DTPROGENTAF' => $dtprogentaf_NOVA,
            'CDFORNECED' => $itemautf->getCdforneced(),
            'CDSEGMARCA' => $itemautf->getCdsegmarca(),
            'NRPARCITMAF' => $itemautf->getNrparcitmaf(),
            'DTEFETENTAF' => $itemautf->getDtefetentaf(),
            'QTPROGENTAF' => $qtprogentaf_NOVA,
            'VRPREUNIAF' => $itemautf->getVrpreuniaf(),
            'VRPERDESAF' => $itemautf->getVrperdesaf(),
            'QTEFETENTRAF' => $itemautf->getQtefetentraf(),
            'CDEMPRESA' => $itemautf->getCdempresa(),
            'IDTIPEMITMAF' => $itemautf->getIdtipemitmaf(),
            'VRPRECECOAF' => $itemautf->getVrprececoaf(),
            'VRPRECCOTAF' => $itemautf->getVrpreccotaf(),
            'IDBAIXITMAF' => $itemautf->getIdbaixitmaf(),
            'NRLANCTONF' => $itemautf->getNrlanctonf(),
            'TXOBSITMAF' => $itemautf->getTxobsitmaf(),
            'VRDESCITMAF' => $itemautf->getVrdescitmaf(),
            'NRLOTEESTQ' => $itemautf->getNrloteestq(),
            'NRSUBLOTE' => $itemautf->getNrsublote(),
            'CDPRODPRODAF' => $itemautf->getCdprodprodaf(),
            'DTENTRITMSOL' => $itemautf->getDtentritmsol(),
            'DSINCONSITAF' => $itemautf->getDsinconsitaf(),
            'IDMOTPRESCAF' => $itemautf->getIdmotprescaf(),
            'QTVOLUITEMAF' => $itemautf->getQtvoluitemaf(),
            'IDTIPOLANITM' => self::IDTIPOLANITM,
            'CDFILCOTITAF' => $itemautf->getCdfilcotitaf(),
            'NRCOTPREITAF' => $itemautf->getNrcotpreitaf(),
            'CDMARCOTITAF' => $itemautf->getCdmarcotitaf(),
            'CDREGCOTITAF' => $itemautf->getCdregcotitaf(),
            'CDFORCOTITAF' => $itemautf->getCdforcotitaf(),
            'CDPROCOTITAF' => $itemautf->getCdprocotitaf(),
            'NRLISTAITAF' => $itemautf->getNrlistaitaf(),
            'IDTPLANITEAF' => $itemautf->getIdtplaniteaf()
        );
        
        return $params;
    }
    
    /**
     * Esse condição foi colocada para permitir que a GR altere a
	 * data de entrega (para uma já existente) sem seja feito o remanejamento,
	 * pois ao remanejar uma AF de antecipação, a data de utilzação do fica com
	 * a maior data. Processo visto in loco dna subida de versão do cliente para a 6.02. Apenas a GR consegue alterar a data
	 * 
	 * Foi retirado a obrigação de se realizar o remanejamento para efetuar a troca da data de entrega do item da AF
    */
    public function verificaExisteAf($cdfildest, $cdfillanc, $nreqapuraf, $cdproduto, $dtprogentaf, $idgeradtent) {
        $idsisdos = $this->sup05100AnaliseAFService->retornaIdsisdos();
        $existeItemAf = $this->sup05100AnaliseAFService->retornaExisteItemAF($cdfildest, $cdfillanc, $nreqapuraf, $cdproduto, $dtprogentaf);
        $mensagem = self::MENSAGENS['AF_EXISTENTE_NA_DATA']; // 'Operação bloqueada! <br> Favor realizar o remanejamento de AF, pois já existe uma AF para esse item na data informada.';
        
        if($existeItemAf) {
            if($idsisdos == '3') {
                if($idgeradtent == 'N') {
                    throw new \Exception($mensagem);
                }
            }
        }
    }
    
    public function retornaItemautf($listaItensAF, $itemNovo) {
        foreach($listaItensAF as $itemAF) {
            if($itemAF->getCdproduto() == $itemNovo['CDPRODUTO'] && $itemAF->getCdmarca() == $itemNovo['CDMARCA'] && self::formataDataString($itemAF->getDtprogentaf()) == $itemNovo['DTPROGENTAF']) {
                return $itemAF;
            }
        }
        
        return false;
    }
    
    public function insereMovopaf($row, $idoperamovaf, $dtprogentaf = null, $dsalteraf = null, $dsjustopermov = null) {
        if($idoperamovaf == 'A') {
            if($row['QTPROGENTAF_OLD'] <> $row['QTPROGENTAF']) {
                $dsalteraf = self::ALTER_QTDE_AF . $row['QTPROGENTAF_OLD'] . self::PARA . $row['QTPROGENTAF'];
                $dsjustopermov = self::ALTER_QTDE_ANALISE;
                
                $params = self::montaParamsInserirMovopaf($row, $row['DTPROGENTAF_OLD'], $dsalteraf, $dsjustopermov, $idoperamovaf);
                $this->sup05100AnaliseAFService->insereMovopaf($params);
            }
            
            if($row['DTPROGENTAF_OLD'] <> $row['DTPROGENTAF']) {
                $dsalteraf = self::ALTER_DTENT_AF . $row['DTPROGENTAF_OLD'] . self::PARA . $row['DTPROGENTAF'];
                $dsjustopermov = self::ALTER_DTENT_ANALISE;
                
                $params = self::montaParamsInserirMovopaf($row, $row['DTPROGENTAF'], $dsalteraf, $dsjustopermov, $idoperamovaf);
                $this->sup05100AnaliseAFService->insereMovopaf($params);
            } 
        } else {
            $params = self::montaParamsInserirMovopaf($row, $dtprogentaf, $dsalteraf, $dsjustopermov, $idoperamovaf);
            $this->sup05100AnaliseAFService->insereMovopaf($params);
        }
    }
    
    public function montaParamsInserirMovopaf($row, $dtprogentaf, $dsalteraf, $dsjustopermov, $idoperamovaf) {
        $params = array(
            'DTPROGENTAF' => $dtprogentaf,
            'DSALTERAF' => $dsalteraf,
            'DSJUSTOPERMOV' => $dsjustopermov,
            'IDOPERAMOVAF' => $idoperamovaf,
            'IDUTATUALIZASOL' => $row['ATUSOLICITACMP'] ? 'S' : 'N',
            'CDFILDEST' => $row['CDFILDEST'],
            'CDFILLANC' => $row['CDFILLANC'],
            'NRAUTOFORN' => $row['NRAUTOFORN'],
            'CDPRODUTO' => $row['CDPRODUTO'],
            'CDMARCA' => $row['CDMARCA']
        );
        
        return $params;
    }
    
    /********************************************
     *              Conclusão de AF             *
     ********************************************/
    
    public function acionaConcluirAF(DTO\Request\Row $request, DTO\Response $response) 
    {
        ini_set('max_execution_time', 9999);
        try {
            $row = $request->getRow();
            
            $afsApuracao = $row['ALL_DATA_APURACAO'];
            $currentRow = $row['CURRENT_ROW'];
            $dadosApuracao = $row['DADOS_APURACAO'];
            
            if (self::isApuracaoEmAnalise($dadosApuracao['IDSITAPUPREC'])) {
                $dadosRelatorio = $row['DADOS_RELATORIO'];
                $dadosEmail = $row['DADOS_EMAIL'];
                $afsSelecionadas = $row['AFS_SELECIONADAS'];
        
                $afsConsistentes = array();
                $afsInconsistentes = false;
                $listaCdforneced = array();
                $novaApuracao = false;
                
                if((is_array($afsSelecionadas) && is_array($afsApuracao)) && count($afsSelecionadas) && count($afsSelecionadas) <> count($afsApuracao)) {
                    $novaApuracao = true;
                    $afsConcluir = $afsSelecionadas;
                } else {
                    $afsConcluir = $afsApuracao;
                
                    if(isset($currentRow['NMFANTFORN'])) {
                        if(is_array($currentRow['NMFANTFORN']) && count($currentRow['NMFANTFORN'])) {
                            $selectedApuracao = array();
                            
                            foreach($currentRow['NMFANTFORN'] as $forn) {
                                $row = array_filter($afsConcluir, function($nrApuracao) use ($forn) {
                                    return $nrApuracao['CDFORNECED'] === $forn;
                                });
                                $selectedApuracao = array_merge($selectedApuracao, $row);
                            }
                            
                            $afsConcluir = $selectedApuracao;
                        }
                    }
                    
                    if(isset($currentRow['NMFILIAL'])) {
                        if(is_array($currentRow['NMFILIAL']) && count($currentRow['NMFILIAL'])) {
                            $selectedApuracao = array();
                            
                            foreach($currentRow['NMFILIAL'] as $filial) {
                                $row = array_filter($afsConcluir, function($nrApuracao) use ($filial) {
                                    return $nrApuracao['CDFILDEST'] === $filial;
                                });
                                $selectedApuracao = array_merge($selectedApuracao, $row);
                            }
                            
                            $afsConcluir = $selectedApuracao;
                        }
                    }
                    
                    if(isset($currentRow['NRAUTOFORN'])) {
                        if(is_array($currentRow['NRAUTOFORN']) && count($currentRow['NRAUTOFORN'])) {
                            $selectedApuracao = array();
                            
                            foreach($currentRow['NRAUTOFORN'] as $af) {
                                $row = array_filter($afsConcluir, function($nrApuracao) use ($af) {
                                	return $nrApuracao['NRAUTOFORN'] === $af;
                                });
                                $selectedApuracao = array_merge($selectedApuracao, $row);
                            }
                            
                            $afsConcluir = $selectedApuracao;
                        }
                    }
                    
                    $novaApuracao = count($afsApuracao) !== count($afsConcluir);
                }
                
                foreach ($afsConcluir as $af) {
                    if(self::isAfInconsistenteOuForaDolimite($af)) {
                        $novaApuracao = true;
                        $afsInconsistentes = true;
                    } else {
                        array_push($afsConsistentes, $af);
                        array_push($listaCdforneced, $af['CDFORNECED']);
                    }
                }
                
                if(is_array($afsApuracao) && !count($afsConsistentes)) {
                    throw new \Exception(self::MENSAGENS['N_Ha_AF_Cons']);
                }
                
                if ($novaApuracao) {
                    $nrApuracao = self::novaApuracao($dadosApuracao, $afsConsistentes);
                } else {
                    $nrApuracao = $dadosApuracao['NRAPURPREC'];
                }
                
                if($afsInconsistentes){
                    $this->logEnvioAF['AF_INCONSIST'] = $afsInconsistentes;
                    $this->logEnvioAF['NEW_NRAPURPREC'] = $nrApuracao;
                }
                
                if($dadosEmail) {
                    self::concluirAF('3', $nrApuracao, $dadosApuracao['CDFILLANC'], $dadosApuracao['CDFILCENTCMP']);
                    self::enviarAFporEmail($listaCdforneced, $nrApuracao, $dadosApuracao, $dadosRelatorio, $dadosEmail, $afsConcluir); 
                } else {
                    self::concluirAF('2',$nrApuracao, $dadosApuracao['CDFILLANC'], $dadosApuracao['CDFILCENTCMP']);
                    if($dadosRelatorio['EMITIR_AF'] == 'S') {
                        self::emitirRelatorioAF($listaCdforneced, $nrApuracao, $dadosApuracao, $dadosRelatorio, $afsConcluir);
                    }
                }

                $response->addMessage(new Message($this->logEnvioAF, 'W'));
            }
            
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function enviarAFporEmail($listaCdforneced, $nrApuracao, $dadosApuracao, $dadosRelatorio, $dadosEmail, $afsConcluir, $isResend = false) 
    {
        $listaFornecedoresUnicos = array_unique($listaCdforneced);
        $dadosEmailFornecedores = $this->sup05100AnaliseAFService->retornaDadosEmail($dadosApuracao);
        
        if(!empty($dadosEmailFornecedores)) {
            foreach($listaFornecedoresUnicos as $codigoFornecedor) {
                $descricaoEnvioAf = null;
                $relatoriosGerados = [];
                
                $emailFornecedores = array_filter($dadosEmailFornecedores, function($fornecedor) use ($codigoFornecedor) {
                	return $fornecedor['CDFORNITMAP'] === $codigoFornecedor;
                });
                
                /**
                 * Optou-se por utilizar o array_pop em vez de acessar a posição
                 * 0 do array pois, devido às chaves serem presevadas pela
                 * função array_filter, realizar essa operação pode ocasionar
                 * Undefined offset, já que o item encontrado pode estar em
                 * posições diferentes de 0.
                 * 
                 * https://www.php.net/manual/pt_BR/function.array-filter.php
                 */
                
                $dadoFornecedor = array_pop($emailFornecedores);
                
                $possuiEmail = self::possuiEmail($dadoFornecedor);
                
                $isEmailEnviado = false;
                
                if ($dadoFornecedor['IDTPENVAFFOR'] === '1') {
                    if($possuiEmail) {
                        
                        $retornoGerarRelatoriosAFs = self::gerarRelatoriosAFs($dadoFornecedor, $dadosApuracao, $dadosRelatorio, $dadoFornecedor['IDTPENVAFNORMAL'] === 'E', $nrApuracao, $afsConcluir, $codigoFornecedor);
                        $relatoriosGerados = $retornoGerarRelatoriosAFs['RELATORIOS'];
                        $numeroAFConcluidas = $retornoGerarRelatoriosAFs['NR_AF_CONCLUIDAS'];
                        $isEmailEnviado = self::enviarRelatoriosPorEmail($dadosEmail, $dadoFornecedor, $dadosApuracao, $nrApuracao, $dadosRelatorio, $relatoriosGerados);
                        $idconcenvioaf = $isEmailEnviado ? 'S' : 'N';
                        self::enviarAfFornecedor($codigoFornecedor, $nrApuracao, $dadosApuracao, $dadosRelatorio, $idconcenvioaf, $descricaoEnvioAf, $listaCdforneced, $numeroAFConcluidas);
                        
                    } else {
                        
                        $retornoGerarRelatoriosAFs = self::gerarRelatoriosAFs($dadoFornecedor, $dadosApuracao, $dadosRelatorio, false, $nrApuracao, $afsConcluir, $codigoFornecedor);
                        $relatoriosGerados = $retornoGerarRelatoriosAFs['RELATORIOS'];
                        $numeroAFConcluidas = $retornoGerarRelatoriosAFs['NR_AF_CONCLUIDAS'];
                        $idconcenvioaf = 'N';
                        self::enviarAfFornecedor($codigoFornecedor, $nrApuracao, $dadosApuracao, $dadosRelatorio, $idconcenvioaf, $descricaoEnvioAf, $listaCdforneced, $numeroAFConcluidas);
                    }
                }
                
                self::addLogEnvioAF($dadoFornecedor, $isEmailEnviado, $possuiEmail, $dadosRelatorio['EMITIR_AF'], $relatoriosGerados);
            }
            
        } else {
            
            $this->logEnvioAF['ERRO'][] = self::MENSAGENS['Linha_LogAF_REENVIO_ERRO'];
        }    
    }
    
    private function enviarRelatoriosPorEmail($dadosEmail, $dadoFornecedor, $dadosApuracao, $nrApuracao, $dadosRelatorio, $relatoriosGerados) 
    {
        if (empty($relatoriosGerados) && $dadoFornecedor['IDTPENVAFNORMAL'] === 'E') {
            return false;
        }
        
        $paramsEmail = $this->retornaParametrosEmail($dadosEmail, $dadoFornecedor, $dadosApuracao, $nrApuracao, $dadosRelatorio, $relatoriosGerados);
        return $this->reportController->enviaEmail($paramsEmail);
    }
    
    private function retornaParametrosEmail($dadosEmail, $dadoFornecedor, $dadosApuracao, $nrApuracao, $dadosRelatorio, $relatoriosGerados) {
        $dadosEmailOperadorLogado = $this->sup05100AnaliseAFService->retornaDadosEmailOperadorLogado();
        $fornEnvia = $this->sup05100AnaliseAFService->adqFornEnvia($dadosApuracao['CDFILLANC'], $nrApuracao);
        
        return array(
            'MENSAGEM'          => $dadosEmail['MENSAGEM'],
            'ASSUNTO'           => $dadosEmail['ASSUNTO'],
            'SENDCOPY'          => $dadosEmail['enviarCopiaOperador'],
            'NOME_REMETENTE'    => $dadosEmailOperadorLogado['NMOPERADOR'],
            'EMAIL_REMETENTE'   => $dadosEmailOperadorLogado['DSEMAILOPER'],
            'NMFANTFORN'        => $dadoFornecedor['NMFANTFORN'],
            'RELATORIOS'        => $relatoriosGerados,
            'DSEMAILCOT'        => $dadoFornecedor['DSEMAILCOT'],
            'DSEMAILCOTCOP'     => $dadoFornecedor['DSMAILCOTCOP'],
            'DSLINKECOMMERCE'   => $this->sup05100AnaliseAFService->retornaLinkPortalEcommerce($dadosApuracao['CDFILCENTCMP']),
            'TYPE'              => $dadosRelatorio['EXTENSAO'],
            'SGESTADO'          => $fornEnvia['SGESTADO'],
            'DTINIAPUPREC'      => $dadosApuracao['DTINIAPUPREC'],
            'DTFINAPUPREC'      => $dadosApuracao['DTFINAPUPREC'],
            'CDFORNECED'        => $dadoFornecedor['CDFORNECED'],
            'NRSEQAPURAF'       => $nrApuracao,
        );
    }
    
    public function emitirRelatorioAF($listaCdforneced, $nrApuracao, $dadosApuracao, $dadosRelatorio, $afsConcluir) 
    {
        $listaFornecedoresUnicos = array_unique($listaCdforneced);
        $dadosAFFornecedores = $this->sup05100AnaliseAFService->retornaDadosAF($nrApuracao, $dadosApuracao['CDFILCENTCMP'], $dadosApuracao['DTINIAPUPREC'], $dadosApuracao['DTFINAPUPREC']);
        
        if(!empty($dadosAFFornecedores)) {
            foreach($listaFornecedoresUnicos as $codigoFornecedor) {
                $descricaoEnvioAf = null;
                
                $emailFornecedores = array_filter($dadosAFFornecedores, function($fornecedor) use ($codigoFornecedor) {
                	return $fornecedor['CDFORNITMAP'] === $codigoFornecedor;
                });
                
                /**
                 * Optou-se por utilizar o array_pop em vez de acessar a posição
                 * 0 do array pois, devido às chaves serem presevadas pela
                 * função array_filter, realizar essa operação pode ocasionar
                 * Undefined offset, já que o item encontrado pode estar em
                 * posições diferentes de 0.
                 * 
                 * https://www.php.net/manual/pt_BR/function.array-filter.php
                 */
                $dadoFornecedor = array_pop($emailFornecedores);
                
                $retornoGerarRelatoriosAFs = self::gerarRelatoriosAFs($dadoFornecedor, $dadosApuracao, $dadosRelatorio, true, $nrApuracao, $afsConcluir, $codigoFornecedor);
                  
                $relatoriosGerados = $retornoGerarRelatoriosAFs['RELATORIOS'];
                $numeroAFConcluidas = $retornoGerarRelatoriosAFs['NR_AF_CONCLUIDAS'];
                
                self::addLogEnvioAF($dadoFornecedor, false, false, $dadosRelatorio['EMITIR_AF'], $relatoriosGerados);
            }
        }
    }
    
    public function possuiEmail($dadosEmail) {
        return !empty($dadosEmail['DSEMAILCOT']);
    }
    
    public function gerarRelatoriosAFs($dadoFornecedor, $dadosApuracao, $dadosRelatorio, $makeReport, $nrApuracao, $afsConcluir, $codigoFornecedor) {
        $retorno = array( 'RELATORIOS' => array(), 'NR_AF_CONCLUIDAS' => array() );
        $urlRelatorio = '';

        if(isset($dadosApuracao['NRAUTOFORN']) && is_array($dadosApuracao['NRAUTOFORN']) && count($dadosApuracao['NRAUTOFORN'])) {
            $nrAutoForn = join("_", array_unique(array_map(function($row) {
                return $row;
            }, $dadosApuracao['NRAUTOFORN'])));
        } else {
            $nrAutoForn = null;    
        }
        
        if ($makeReport) {
            $paramsRelatorio = $this->retornaParametrosRelatorio($nrApuracao, $dadosRelatorio, $dadoFornecedor, $nrAutoForn, $dadosApuracao);
            $urlRelatorio = $this->reportController->geraRelatorio($paramsRelatorio);
        }
        
        if (!empty($urlRelatorio)) {
            $retorno['RELATORIOS'][] = $urlRelatorio;
        }
        
        $autoforne = array_filter($afsConcluir, function($fornecedor) use ($codigoFornecedor) {
        	return $fornecedor['CDFORNECED'] === $codigoFornecedor;
        });
                        
        foreach($autoforne as $af) {
            $retorno['NR_AF_CONCLUIDAS'][] = $af['NRAUTOFORN'];
        }
                        
        return $retorno;
    }
    
    private function retornaParametrosRelatorio($nrApuracao, $dadosRelatorio, $dadoFornecedor, $nrAutoForn, $dadosApuracao) 
    {
        $nomeRelatorioSplit = explode('.', $dadosRelatorio['NOME_RELATORIO']);
        /**
         * 
         * Criar opções de imprimir observaçõe do produto e da unidade
         * 
        **/
        return array(
            '__report_name'     => $nomeRelatorioSplit[0],
            'QR2TYPE'           => $dadosRelatorio['EXTENSAO'],
            'STRATEGY'          => $nomeRelatorioSplit[1],
            'TIPO'              => 0,
            'CDFORNECED'        => $dadoFornecedor['CDFORNITMAP'],
            'CDFILDEST'         => $dadoFornecedor['CDFILDEST'],
            'NRAUTOFORN'        => $nrAutoForn,
            'NRSEQAPURAF'       => $nrApuracao,
            'CDFILLANC'         => $dadoFornecedor['CDFILLANC'],
            'NRSEQENVIOAFORN'   => 'T',
            'NRLOGPLANITEMAF'   => null,
            'CDCOMPRADOR'       => $dadosApuracao['CDCOMPRADOR'],
            'IMPRIMEOBS'        => 'S',
            'IMPRIMEOBSPROD'    => 'S',
            'NRORG'             => $this->environment->getNrOrg()
        );
    }
    
    public function concluirAF($tipo, $nrApuracao, $codigoFilialLancamento, $codigoCentralCompras) {
        $this->sup05100AnaliseAFService->concluirApuracaoPreco($tipo, $codigoCentralCompras, $nrApuracao);
        $this->sup05100AnaliseAFService->concluirNormal($nrApuracao, $codigoFilialLancamento);
        $this->sup05100AnaliseAFService->concluirEDI($nrApuracao, $codigoFilialLancamento);
    }
    
    public function isAfInconsistenteOuForaDolimite($af) {
        return $af['IDSITUATUAAF'] == '1' || $af['IDAFFORALIM'] == 'S';
    }
    
    public function novaApuracao($dadosApuracao, $afsConsistentes) {
        $novoCodigo = $this->sup05100AnaliseAFService->novoCodigo('APURPRECO' . $dadosApuracao['CDFILCENTCMP']);
        
        $this->sup05100AnaliseAFService->insereApuracao($novoCodigo, $dadosApuracao['NRAPURPREC'], $dadosApuracao['CDFILCENTCMP']);

        $this->sup05100AnaliseAFService->insereItemApuracao($novoCodigo, $dadosApuracao['NRAPURPREC'], $dadosApuracao['CDFILCENTCMP']);
        
        foreach($afsConsistentes as $af) {
            $this->sup05100AnaliseAFService->atualizaAutoforneApuracao($novoCodigo, $af['CDFILDEST'], $af['CDFILLANC'], $af['NRAUTOFORN']);
        }
        
        return $novoCodigo;
    }
    
    public function enviarAfFornecedor($codigoFornecedorEmail, $nrApuracao, $dadosApuracao, $dadosRelatorio, $idconcenvioaf, $descricaoEnvioAf, $listaFornecedoresAfsConsistentes, $numeroAFConcluidas) 
    {
        $nrEnvio = $this->sup05100AnaliseAFService->novoCodigo('ENVIOAFFORN');
        
        $qtdAfsConcluidasDoFornecedor = count(array_filter($listaFornecedoresAfsConsistentes, function($codigoFornecedor) use ($codigoFornecedorEmail) {
        	return $codigoFornecedor == $codigoFornecedorEmail;
        }));
        
        if(!isset($dadosApuracao['ISRESENDEMAIL'])) {
            $this->sup05100AnaliseAFService->insereEnviaAf($nrEnvio, $codigoFornecedorEmail, $descricaoEnvioAf, $nrApuracao, $dadosApuracao['DTINIAPUPREC'], $dadosApuracao['DTFINAPUPREC'], $qtdAfsConcluidasDoFornecedor, 'S', $dadosRelatorio['EXTENSAO'], $idconcenvioaf);
        }
        
        foreach($numeroAFConcluidas as $nrAutorizacaoFornecimento) {
            $this->sup05100AnaliseAFService->atualizaSeqEnvioAf($nrApuracao, $nrEnvio, $codigoFornecedorEmail, $dadosApuracao, $nrAutorizacaoFornecimento, $idconcenvioaf);
        }
    }
    
    public function verificaClasseConclusaoAf($classe, $dadosEmail, $dtEntrega) {
        $idconsolapuefet = $this->sup05100AnaliseAFService->retornaIdconsolapuefet($dadosEmail['CDFILLANC'], $dadosEmail['NRAPURPREC']);
        
        $dtiniapuprec = DateUtil::getDataDeString($dadosEmail['DTINIAPUPREC'], DateUtil::FORMATO_BRASILEIRO_DATAHORA);
        
        if($idconsolapuefet == 'S') {
            $classe = 'E';
        } else if(DateUtil::comparaDatas($dtEntrega, $dtiniapuprec)) {
            $classe = 'A';
        }
        
        return $classe;
    }
    
    public function addLogEnvioAF($dadoFornecedor, $emailEnviado, $possuiEmail, $emitirRel, $relatoriosGerados) {
        if($emitirRel == 'N'){
            if ($dadoFornecedor['IDTPENVAFFOR'] !== '1') {
                $mensagem  = sprintf(self::MENSAGENS['Linha_LogAF_AGRUP_ERRO'], $dadoFornecedor['NMFANTFORN']);
                $mensagem .= ' ' . self::MENSAGENS['MAIL_NOT_SENDED_IDTPENVAFFOR'];
                    
                $this->logEnvioAF['ERRO'][] = $mensagem;
            } else {
                if ($dadoFornecedor['IDTPENVAFNORMAL'] == 'P') {
                    if($emailEnviado) {
                        $mensagem  = sprintf(self::MENSAGENS['Linha_LogAF_PORTAL_OK'], $dadoFornecedor['NMFANTFORN']);
                        $mensagem .= !$emailEnviado ? ' ' . self::MENSAGENS['Email_S_AF_ERRO'] : '';
                        $mensagem .= !$possuiEmail ? ' ' . self::MENSAGENS['FORN_SEM_EMAIL'] : '';
                        
                        $this->logEnvioAF['SUCESSO'][] = $mensagem;
                    } else {
                        $mensagem  = sprintf(self::MENSAGENS['Linha_LogAF_PORTAL_OK'], $dadoFornecedor['NMFANTFORN']);
                        $mensagem .= !$emailEnviado ? ' ' . self::MENSAGENS['Email_S_AF_ERRO'] : '';
                        $mensagem .= !$possuiEmail ? ' ' . self::MENSAGENS['FORN_SEM_EMAIL'] : '';
                        
                        $this->logEnvioAF['ERRO'][] = $mensagem;
                    }
                } else {
                    if ($emailEnviado) {
                        $this->logEnvioAF['SUCESSO'][] = sprintf(self::MENSAGENS['Linha_LogAF_AGRUP_OK'], $dadoFornecedor['NMFANTFORN']);
                    } else {
                        $mensagem  = sprintf(self::MENSAGENS['Linha_LogAF_AGRUP_ERRO'], $dadoFornecedor['NMFANTFORN']);
                        $mensagem .= !$possuiEmail ? ' ' . self::MENSAGENS['FORN_SEM_EMAIL'] : '';
                        
                        $this->logEnvioAF['ERRO'][] = $mensagem;
                    }
                }
            }
            
            $this->logEnvioAF['REPORT'][] = $relatoriosGerados;
        } else {
            if($emitirRel == 'S'){
                if (!empty($relatoriosGerados)) {
                    $this->logEnvioAF['SUCESSO'][] = $relatoriosGerados;    
                } else {
                    $this->logEnvioAF['ERRO'][] = self::MENSAGENS['NAO_HA_DADOS'];
                }   
            }  
        }
    }
    
    /********************************************
     *               Remanejar AF               *
     ********************************************/
     
    public function acionaRemanejarAF(DTO\Request\Row $request, DTO\Response $response)
    {
        try {
            $dadosRemaneja = $request->getRow();
            
            if (self::isApuracaoEmAnalise($dadosRemaneja['IDSITAPUPREC'])) {
                
                try {
                    
                    $this->sup05100AnaliseAFService->iniciaConexao();
                    
                    $result = array();
        
                    foreach ($dadosRemaneja['AF_ORIGEM'] as $afOrigem) {
                        $data = array('PRODUCTS' => array());
                        
                        $remaneja = $this->sup05100AnaliseAFService->remanejaProcedure($afOrigem, $dadosRemaneja['AF_DESTINO']);
                        $fornecedor = $this->sup05100AnaliseAFService->getFornecedor($dadosRemaneja['AF_DESTINO']);
                        
                        if($remaneja['P_RESULT_ERROS'] !== 'NNN') {
                        
                            $str_result = explode('_', $remaneja['P_RESULT_ERROS']);
                            
                            $data['NRAUTOFORN'] = $afOrigem['NRAUTOFORN'];
                            $data['DTPROGENTAF'] = $afOrigem['DTPROGENTAF'];
                            
                            $products = array();
                            foreach ($str_result as $row) {
                                if(!empty($row)) {
                                    $produto = $this->sup05100AnaliseAFService->getProduto(substr($row, 0, 10));
                                    $products['ERROR'] = substr($row, 10, 13);
                                    $products['CDARVPROD'] = $produto[0]['CDARVPROD'];
                                    $products['CDPRODUTO'] = $produto[0]['CDPRODUTO'];
                                    $products['NMPRODUTO'] = $produto[0]['NMPRODUTO'];
                                    
                                    $products['NMFANTFORN']   = $fornecedor[0]['NMFANTFORN'];
                                    $products['NMRAZSOCFORN'] = $fornecedor[0]['NMRAZSOCFORN'];
                                    
                                    array_push($data['PRODUCTS'], $products);
                                }
                            }
                        
                            array_push($result, $data);
                        } else {
                            array_push($result, $remaneja);
                        }
                    }    
                    
                    $this->sup05100AnaliseAFService->realizaFlush();
                    $this->sup05100AnaliseAFService->realizaCommit();
                    
                    $response->addMessage(new Message($result, 'W'));
                                
                } catch(\Exception $e) {
                    $this->sup05100AnaliseAFService->realizaRollback();
                    $response->setError(new Error($e->getMessage(), $e->getCode()));
                }
            }
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    private function acionaProcedureRemanejarAF($response, $dadosRemaneja, $newProduct = null)
    {
        $result = array();
        
        foreach ($dadosRemaneja['AF_ORIGEM'] as $afOrigem) {
            $data = array('PRODUCTS' => array());
            
            $remaneja = $this->sup05100AnaliseAFService->remanejaProcedure($afOrigem, $dadosRemaneja['AF_DESTINO'], true, $newProduct);
            $fornecedor = $this->sup05100AnaliseAFService->getFornecedor($dadosRemaneja['AF_DESTINO']);
            
            if($remaneja['P_RESULT_ERROS'] !== 'NNN') {
            
                $str_result = explode('_', $remaneja['P_RESULT_ERROS']);
                
                $data['NRAUTOFORN'] = $afOrigem['NRAUTOFORN'];
                $data['DTPROGENTAF'] = $afOrigem['DTPROGENTAF'];
                
                $products = array();
                foreach ($str_result as $row) {
                    if(!empty($row)) {
                        $produto = $this->sup05100AnaliseAFService->getProduto(substr($row, 0, 10));
                        $products['ERROR'] = substr($row, 10, 13);
                        $products['CDARVPROD'] = $produto[0]['CDARVPROD'];
                        $products['CDPRODUTO'] = $produto[0]['CDPRODUTO'];
                        $products['NMPRODUTO'] = $produto[0]['NMPRODUTO'];
                        
                        $products['NMFANTFORN']   = $fornecedor[0]['NMFANTFORN'];
                        $products['NMRAZSOCFORN'] = $fornecedor[0]['NMRAZSOCFORN'];
                        
                        array_push($data['PRODUCTS'], $products);
                    }
                }
                
                array_push($result, $data);
            } else {
                array_push($result, $remaneja);
            }
        }    
        
        $response->addMessage(new Message($result, 'W'));
        
        return $response;
    }
    
    public function acionaRemanejarItensAF(DTO\Request\Row $request, DTO\Response $response)
    {
        try {
            $dadosRemaneja = $request->getRow();
            
            if (self::isApuracaoEmAnalise($dadosRemaneja['IDSITAPUPREC'])) {
                
                try {
                    
                    $this->sup05100AnaliseAFService->iniciaConexao();
                    $reponse = $this->acionaProcedureRemanejarAF($response, $dadosRemaneja);
                    
                    $this->sup05100AnaliseAFService->realizaFlush();
                    $this->sup05100AnaliseAFService->realizaCommit();
                                
                } catch(\Exception $e) {
                    $this->sup05100AnaliseAFService->realizaRollback();
                    $response->setError(new Error($e->getMessage(), $e->getCode()));
                }
            }
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function acionaRemanejarTrocarProduto(DTO\Request\Row $request, DTO\Response $response)
    {
        try {
            $dadosRemaneja = $request->getRow();
            
            if (self::isApuracaoEmAnalise($dadosRemaneja['IDSITAPUPREC'])) {
                
                try {
                    
                    $this->sup05100AnaliseAFService->iniciaConexao();
                    $reponse = $this->acionaProcedureRemanejarAF($response, $dadosRemaneja, $dadosRemaneja['ITEM_DESTINO']);
                    
                    $this->sup05100AnaliseAFService->realizaFlush();
                    $this->sup05100AnaliseAFService->realizaCommit();
                                
                } catch(\Exception $e) {
                    $this->sup05100AnaliseAFService->realizaRollback();
                    $response->setError(new Error($e->getMessage(), $e->getCode()));
                }
            }
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function excluiNFContaPagar($produto) {
        
        $quantidadeItemNF = $this->sup05100AnaliseAFService->retornaQuantidadeItemNF($produto['CDFILDEST'], $produto['CDFILLANC'], $produto['NRAUTOFORN']);
        
        if($quantidadeItemNF == '1') {
            
            $this->contaPagar->excluiNFContaPagar($produto['CDFILDEST'], $produto['CDFILLANC'], $produto['NRAUTOFORN'], false);
            
        }
        
    }
    
    /********************************************
     *                Excluir AF                *
     ********************************************/
    
    public function acionaExcluirAF(DTO\Request\Row $request, DTO\Response $response) 
    {
        ini_set('max_execution_time', 9999);
        $row = $request->getRow();
        
        try {
            if (empty($row['AFS_SELECIONADAS'])) {
                $this->validaCompradorExcluirAF($row);
                $mensagem = $this->excluiAF($row);
            } else {
                $mensagem = $this->delAF($row);
            }
            
            if (!empty($mensagem)) {
                $response->addMessage(new Message($mensagem));
            }
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function delAF($row) 
    {
        try {
            $cdsAux = $row['AFS_SELECIONADAS'];
            
            foreach ($cdsAux as $af) {
                $this->contaPagar->excluiNFContaPagar($af['CDFILDEST'], $af['CDFILLANC'], $af['NRAUTOFORN'], false);
                
                $paramsAcmExcluiAFChave = array(
                    'P_CDFILDEST'   => $af['CDFILDEST'],
                    'P_CDFILLANC'   => $af['CDFILLANC'],
                    'P_NRAUTOFORN'  => $af['NRAUTOFORN']
                );
                $this->sup05100AnaliseAFService->acmExcluiAF_Chave($paramsAcmExcluiAFChave);
                
                $qtFracionado = $this->sup05100AnaliseAFService->getQtFracionado($af);
                
                if($qtFracionado[0]) {
                    if ($qtFracionado[0]['IDQTFRACIONADO'] == 'S') {
                        $paramsAcmDelItemAutFClone = array(
                            'P_NRSEQAPURAF' => $row['P_NRAPURPREC'],
                            'P_NRAUTOFORN'  => $af['NRAUTOFORN'],
                            'P_CDFILDEST'   => $af['CDFILDEST'],
                            'P_CDFILLANC'   => $af['CDFILLANC']
                        );
                        $this->sup05100AnaliseAFService->acmDelItemAutF_Clone($paramsAcmDelItemAutFClone);
                    }    
                }
                
                
                $paramsAcmDelEnvioAF = array(
                    'P_NRAPURPREC'  => $row['P_NRAPURPREC'],
                    'P_CDFILDEST'   => $af['CDFILDEST'],
                    'P_CDFILLANC'   => $af['CDFILLANC'],
                    'P_CDFORNECED'  => $af['CDFORNECED']
                );
                
                $this->sup05100AnaliseAFService->acmDelEnvioAF($paramsAcmDelEnvioAF);
            }
            
        } catch(\Exception $e) {
            $Result = $e->getMessage();
        }
    }
    
    public function excluiAF($row) {
        $Result = '';
        
        try {
            try {
                if ($row['P_AUTOFORNE'] == 'T') {
                    $paramsAdqTodasAF = array(
                        'P_NRAPURPREC'  => $row['P_NRAPURPREC'], // ParamByName('P_NRAPURPREC' ).AsString := P_NRAPURPREC;
                        'P_CDFORNECED'  => $row['P_CDFORNECED'], // ParamByName('P_CDFORNECED' ).AsString := P_CDFORNECED;
                        'P_CDFILIAL'    => $row['P_CDFILIAL'], // ParamByName('P_CDFILIAL'   ).AsString := P_CDFILIAL;
                        'P_CDCOMPRADOR' => $row['P_CDCOMPRADOR'] // ParamByName('P_CDCOMPRADOR').AsString := P_CDCOMPRADOR;
                    );
                    $todasAF = $this->sup05100AnaliseAFService->adqTodasAF($paramsAdqTodasAF);
                    
                    foreach ($todasAF as $af) {
                        // dtmGeraContaPagar.Exclui_NF_ContaPagar(FieldByName('CDFILDEST' ).AsString,
                        //                                      FieldByName('CDFILLANC' ).AsString,
                        //                                      FieldByName('NRAUTOFORN').AsString,
                        //                                      FILIPROC,
                        //                                      False);
                        $this->contaPagar->excluiNFContaPagar($af['CDFILDEST'], $af['CDFILLANC'], $af['NRAUTOFORN'], false);
                        
                        $paramsAcmDelAutoForne = array(
                            'OLD_CDFILDEST'     => $af['CDFILDEST'], // acmDelAutoForne.Params.ParamByName('OLD_CDFILDEST' ).AsString := FieldByName('CDFILDEST' ).AsString;
                            'OLD_CDFILLANC'     => $af['CDFILLANC'], // acmDelAutoForne.Params.ParamByName('OLD_CDFILLANC' ).AsString := FieldByName('CDFILLANC' ).AsString;
                            'OLD_NRAUTOFORN'    => $af['NRAUTOFORN'] // acmDelAutoForne.Params.ParamByName('OLD_NRAUTOFORN').AsString := FieldByName('NRAUTOFORN').AsString;
                        );
                        $this->sup05100AnaliseAFService->acmDelAutoForne($paramsAcmDelAutoForne);
    
                        //Deleta tabela ENVIOAFFORN
                        $paramsAcmDelEnvioAF = array(
                            'P_NRAPURPREC'  => $af['NRSEQAPURAF'], // ParamByName('P_NRAPURPREC').AsString := FieldByName('NRSEQAPURAF').AsString;
                            'P_CDFILDEST'   => $af['CDFILDEST'], // ParamByName('P_CDFILDEST' ).AsString := FieldByName('CDFILDEST'  ).AsString;
                            'P_CDFILLANC'   => $af['CDFILLANC'], // ParamByName('P_CDFILLANC' ).AsString := FieldByName('CDFILLANC'  ).AsString;
                            'P_CDFORNECED'  => $af['CDFORNECED'] // ParamByName('P_CDFORNECED').AsString := FieldByName('CDFORNECED' ).AsString;
                        );
                        //  Execute;
                        $this->sup05100AnaliseAFService->acmDelEnvioAF($paramsAcmDelEnvioAF);
                    }
                    
                    /**
                     * Este trecho estava comentado no arquivo ao qual foi baseado, no dia 23/04/2019.
                    { with acmTodasAF do begin
                       ParamByName('P_NRAPURPREC').AsString  := P_NRAPURPREC;
                       ParamByName('P_CDFORNECED').AsString  := P_CDFORNECED;
                       ParamByName('P_CDFILIAL').AsString    := P_CDFILIAL;
                       ParamByName('P_CDCOMPRADOR').AsString := P_CDCOMPRADOR;
                       Execute;
                    end;}*/
                    
                } else {
                    $stNrListaAF = self::prox_Lista_AF($row['P_AUTOFORNE']);
                    
                    $listaAF = $this->sup05100AnaliseAFService->adqListaAF(array( 'P_NRLISTAAF' => $stNrListaAF )); // ParamByName('P_NRLISTAAF').AsString  := stNrListaAF;
                    
                    foreach ($listaAF as $af) {
                        // dtmGeraContaPagar.Exclui_NF_ContaPagar(FieldByName('CDFILDEST' ).AsString,
                        //                                      FieldByName('CDFILLANC' ).AsString,
                        //                                      FieldByName('NRAUTOFORN').AsString,
                        //                                      FILIPROC,
                        //                                      False);
                        
                        self::excluiNFContaPagar($af);
                        // ou
                        $this->contaPagar->excluiNFContaPagar($af['CDFILDEST'], $af['CDFILLANC'], $af['NRAUTOFORN'], false);
                        
                        $paramsAcmDelAutoForne = array(
                            'OLD_CDFILDEST'     => $af['CDFILDEST'], // acmDelAutoForne.Params.ParamByName('OLD_CDFILDEST' ).AsString := FieldByName('CDFILDEST' ).AsString;
                            'OLD_CDFILLANC'     => $af['CDFILLANC'], // acmDelAutoForne.Params.ParamByName('OLD_CDFILLANC' ).AsString := FieldByName('CDFILLANC' ).AsString;
                            'OLD_NRAUTOFORN'    => $af['NRAUTOFORN'] // acmDelAutoForne.Params.ParamByName('OLD_NRAUTOFORN').AsString := FieldByName('NRAUTOFORN').AsString;
                        );
                        $this->sup05100AnaliseAFService->acmDelAutoForne($paramsAcmDelAutoForne);
    
                        //Deleta tabela ENVIOAFFORN
                        $paramsAcmDelEnvioAF = array(
                            'P_NRAPURPREC'  => $row['P_NRAPURPREC'], // ParamByName('P_NRAPURPREC').AsString := P_NRAPURPREC;
                            'P_CDFILDEST'   => $af['CDFILDEST'], // ParamByName('P_CDFILDEST' ).AsString := FieldByName('CDFILDEST'  ).AsString;
                            'P_CDFILLANC'   => $af['CDFILLANC'], // ParamByName('P_CDFILLANC' ).AsString := FieldByName('CDFILLANC'  ).AsString;
                            'P_CDFORNECED'  => $row['P_CDFORNECED'] // ParamByName('P_CDFORNECED').AsString := P_CDFORNECED;
                        );
                        $this->sup05100AnaliseAFService->acmDelEnvioAF($paramsAcmDelEnvioAF);
                    }
                    /**
                     * Este trecho estava comentado no arquivo ao qual foi baseado, no dia 23/04/2019.
                    {with acmExcluiAF do begin
                       ParamByName('P_NRLISTAAF').AsString  := stNrListaAF;
                       Execute;
                    end;}*/
                    
                }
                
            } catch (\Exception $e) {
                $Result = $e->getMessage(); // ADOExceptionMessage(E, adcAnaliseAF);
            }
        } finally {
            if (!empty($stNrListaAF)) {
                self::delete_Lista($stNrListaAF);
            }
        }
        
        return $Result;
    }
    
    public function validaCompradorExcluirAF($row) {
        if (!empty($row['P_CDCOMPRADOR'])) {
            $vdComprador = $this->sup05100AnaliseAFService->qryVdComprador( array('P_CDOPERADOR' => $row['P_CDCOMPRADOR'] )); // ParamByName('P_CDOPERADOR').AsString := medCdComprador.Text;
            
            if (empty($vdComprador)) {
                throw new \Exception(self::MENSAGENS['Compr_NCAD']);
            }
        }
    }
     
    /********************************************
     *                Liberar AF                *
     ********************************************/
    
    public function acionaLiberarAFInconsistente(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        
        try {
            if (empty($row['AFS_SELECIONADAS'])) {
                $mensagem = $this->liberaListInconsis($row);
            } else {
                if (self::isApuracaoEmAnalise($row['IDSITAPUPREC'])) {
                    $mensagem = $this->liberAFIncons($row);
                }
            }
            
            if (!empty($mensagem)) {
                $response->addMessage(new Message($mensagem));
            }
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    private function liberAFIncons($row) {
        $stMsg = '';
        
        $afsSelecionadas = $row['AFS_SELECIONADAS'];
        if (!empty($afsSelecionadas)) {
            foreach ($afsSelecionadas as $af) {
                if (($af['IDAFFORALIM'] == 'S') or (!empty($af['IDSITUATUAAF']))) {
                    if ($af['IDAFFORALIM'] == 'S') {
                        $paramsAcmUpAFIncons = array(
                            "P_CDFILDEST"   =>  $af['CDFILDEST'], //P_CDFILDEST; // ParamByName('P_CDFILDEST').AsString  := cdsAuxFornecedor.FieldByName('CDFILDEST').AsString;//P_CDFILDEST;
                            "P_CDFILLANC"   =>  $af['CDFILLANC'], //P_CDFILLANC; // ParamByName('P_CDFILLANC').AsString  := cdsAuxFornecedor.FieldByName('CDFILLANC').AsString;//P_CDFILLANC;
                            "P_NRAUTOFORN"  =>  $af['NRAUTOFORN'], //P_NRAUTOFORN; // ParamByName('P_NRAUTOFORN').AsString := cdsAuxFornecedor.FieldByName('NRAUTOFORN').AsString;//P_NRAUTOFORN;
                            "P_CDGERENTE"   =>  $row['P_CDGERENTE'] // ParamByName('P_CDGERENTE').AsString  := P_CDGERENTE;
                        );
                        $this->sup05100AnaliseAFService->acmUpAFIncons($paramsAcmUpAFIncons);
                    }
                } else {
                    $parametro = $this->sup05100AnaliseAFService->adqParametro(array( 'P_CDFILIAL' => $af['CDFILDEST'] ))[0]; // ParamByName('P_CDFILIAL').AsString := cdsAuxFornecedor.FieldByName('CDFILDEST').AsString;//P_CDFILDEST;
                    
                    if ($parametro['IDAFCOMPLIMGER'] == 'S') {
                        $paramsAdqLimiteGeralGerente = array(
                            'P_CDFILDEST'   => $af['CDFILDEST'], // ParamByName('P_CDFILDEST').AsString  := cdsAuxFornecedor.FieldByName('CDFILDEST').AsString;//P_CDFILDEST;
                            'P_CDFILLANC'   => $af['CDFILLANC'], // ParamByName('P_CDFILLANC').AsString  := cdsAuxFornecedor.FieldByName('CDFILLANC').AsString;//P_CDFILLANC;
                            'P_NRAUTOFORN'  => $af['NRAUTOFORN'], // ParamByName('P_NRAUTOFORN').AsString := cdsAuxFornecedor.FieldByName('NRAUTOFORN').AsString;//P_NRAUTOFORN;
                            'P_CDGERENTE'   => $row['P_CDGERENTE'] // ParamByName('P_CDGERENTE').AsString  := P_CDGERENTE;
                        );
                        $limiteGeralGerente = $this->sup05100AnaliseAFService->adqLimiteGeralGerente($paramsAdqLimiteGeralGerente);
                        
                        if (!empty($limiteGeralGerente)) {
                            $paramsAcmUpAFIncons = array(
                                'P_CDFILDEST'   => $af['CDFILDEST'], // ParamByName('P_CDFILDEST').AsString  := cdsAuxFornecedor.FieldByName('CDFILDEST').AsString;//P_CDFILDEST;
                                'P_CDFILLANC'   => $af['CDFILLANC'], // ParamByName('P_CDFILLANC').AsString  := cdsAuxFornecedor.FieldByName('CDFILLANC').AsString;//P_CDFILLANC;
                                'P_NRAUTOFORN'  => $af['NRAUTOFORN'], // ParamByName('P_NRAUTOFORN').AsString := cdsAuxFornecedor.FieldByName('NRAUTOFORN').AsString;//P_NRAUTOFORN;
                                'P_CDGERENTE'   => $row['P_CDGERENTE'] // ParamByName('P_CDGERENTE').AsString  := P_CDGERENTE;
                            );
                            $this->sup05100AnaliseAFService->acmUpAFIncons($paramsAcmUpAFIncons);
                        } else {
                            $stMsg = self::MENSAGENS['Erro_LibAF'];
                        }
                    } else {
                        $paramsAdqLimiteGerente = array(
                            'P_CDFILDEST'   => $af['CDFILDEST'], // ParamByName('P_CDFILDEST').AsString  := cdsAuxFornecedor.FieldByName('CDFILDEST').AsString;//P_CDFILDEST;
                            'P_CDFILLANC'   => $af['CDFILLANC'], // ParamByName('P_CDFILLANC').AsString  := cdsAuxFornecedor.FieldByName('CDFILLANC').AsString;//P_CDFILLANC;
                            'P_NRAUTOFORN'  => $af['NRAUTOFORN'], // ParamByName('P_NRAUTOFORN').AsString := cdsAuxFornecedor.FieldByName('NRAUTOFORN').AsString;//P_NRAUTOFORN;
                            'P_CDGERENTE'   => $row['P_CDGERENTE'], // ParamByName('P_CDGERENTE').AsString  := P_CDGERENTE;
                            'P_CDOPERADOR'  => $af['CDOPERADOR'], // ParamByName('P_CDOPERADOR').AsString := cdsAuxFornecedor.FieldByName('CDOPERADOR').AsString;
                        );
                        $limiteGerente = $this->sup05100AnaliseAFService->adqLimiteGerente($paramsAdqLimiteGerente);
                        
                        if (!empty($limiteGerente)) {
                            $paramsAcmUpAFIncons = array(
                                'P_CDFILDEST'   => $af['CDFILDEST'], // ParamByName('P_CDFILDEST').AsString  := cdsAuxFornecedor.FieldByName('CDFILDEST').AsString;//P_CDFILDEST;
                                'P_CDFILLANC'   => $af['CDFILLANC'], // ParamByName('P_CDFILLANC').AsString  := cdsAuxFornecedor.FieldByName('CDFILLANC').AsString;//P_CDFILLANC;
                                'P_NRAUTOFORN'  => $af['NRAUTOFORN'], // ParamByName('P_NRAUTOFORN').AsString := cdsAuxFornecedor.FieldByName('NRAUTOFORN').AsString;//P_NRAUTOFORN;
                                'P_CDGERENTE'   => $row['P_CDGERENTE'] // ParamByName('P_CDGERENTE').AsString  := P_CDGERENTE;
                            );
                            $this->sup05100AnaliseAFService->acmUpAFIncons($paramsAcmUpAFIncons);
                        } else {
                            $stMsg = self::MENSAGENS['Erro_LibAF'];
                        }
                    }
                }
            }
        }
        
        return $stMsg;
    }
    
    private function liberaListInconsis($row) {
        $stNrListaAF = '';
        $stMsg = '';
        if ($row['P_AUTOFORNE'] == 'T') {
            $paramsAdqTodasInconsAF = array(
                'P_NRAPURPREC'  => $row['P_NRAPURPREC'], // ParamByName('P_NRAPURPREC').AsString  := P_NRAPURPREC;
                'P_CDFORNECED'  => $row['P_CDFORNECED'], // ParamByName('P_CDFORNECED').AsString  := P_CDFORNECED;
                'P_CDFILIAL'    => $row['P_CDFILIAL'], // ParamByName('P_CDFILIAL').AsString    := P_CDFILIAL;
                'P_CDCOMPRADOR' => $row['P_CDCOMPRADOR'] // ParamByName('P_CDCOMPRADOR').AsString := P_CDCOMPRADOR;
            );
            $todasInconsAF = $this->sup05100AnaliseAFService->adqTodasInconsAF($paramsAdqTodasInconsAF);
            
            foreach($todasInconsAF as $af) {
                $parametro = $this->sup05100AnaliseAFService->adqParametro(array('P_CDFILIAL' => $af['CDFILDEST']))[0]; // ParamByName('P_CDFILIAL').AsString := adqTodasInconsAF.FieldByName('CDFILDEST').AsString;
            
                
                if ($parametro['IDAFCOMPLIMGER'] == 'S') {
                    $paramsAdqLimiteGeralGerente = array(
                        'P_CDFILDEST'   => $af['CDFILDEST'], // adqTodasInconsAF.FieldByName('CDFILDEST').AsString;
                        'P_CDFILLANC'   => $af['CDFILLANC'], // adqTodasInconsAF.FieldByName('CDFILLANC').AsString;
                        'P_NRAUTOFORN'  => $af['NRAUTOFORN'], // adqTodasInconsAF.FieldByName('NRAUTOFORN').AsString;
                        'P_CDGERENTE'   => $row['P_CDGERENTE'] // P_CDGERENTE;
                    );
                    $limiteGeralGerente = $this->sup05100AnaliseAFService->adqLimiteGeralGerente($paramsAdqLimiteGeralGerente);
                    
                    if (!empty($limiteGeralGerente)) {
                        $paramsAcmUpAFIncons = array(
                            'P_CDFILDEST'   => $af['CDFILDEST'], // adqTodasInconsAF.FieldByName('CDFILDEST').AsString;//P_CDFILDEST;
                            'P_CDFILLANC'   => $af['CDFILLANC'], // adqTodasInconsAF.FieldByName('CDFILLANC').AsString;//P_CDFILLANC;
                            'P_NRAUTOFORN'  => $af['NRAUTOFORN'], // adqTodasInconsAF.FieldByName('NRAUTOFORN').AsString;//P_NRAUTOFORN;
                            'P_CDGERENTE'   => $row['P_CDGERENTE'] // P_CDGERENTE;
                        );
                        $this->sup05100AnaliseAFService->acmUpAFIncons($paramsAcmUpAFIncons);
                    } else {
                        $stMsg = self::MENSAGENS['Erro_LibAF'];
                    }
                } else {
                    $paramsAdqLimiteGerente = array(
                        'P_CDFILDEST'   => $af['CDFILDEST'], // adqTodasInconsAF.FieldByName('CDFILDEST').AsString;//P_CDFILDEST;
                        'P_CDFILLANC'   => $af['CDFILLANC'], // adqTodasInconsAF.FieldByName('CDFILLANC').AsString;//P_CDFILLANC;
                        'P_NRAUTOFORN'  => $af['NRAUTOFORN'], // adqTodasInconsAF.FieldByName('NRAUTOFORN').AsString;//P_NRAUTOFORN;
                        'P_CDGERENTE'   => $row['P_CDGERENTE'], // P_CDGERENTE;
                        'P_CDOPERADOR'  => $af['CDOPERADOR'] // adqTodasInconsAF.FieldByName('CDOPERADOR').AsString;
                    );
                    $limiteGerente = $this->sup05100AnaliseAFService->adqLimiteGerente($paramsAdqLimiteGerente);
                    if (!empty($limiteGerente)) {
                        $paramsAcmUpAFIncons = array(
                            'P_CDFILDEST'   => $af['CDFILDEST'], // adqTodasInconsAF.FieldByName('CDFILDEST').AsString;//P_CDFILDEST;
                            'P_CDFILLANC'   => $af['CDFILLANC'], // adqTodasInconsAF.FieldByName('CDFILLANC').AsString;//P_CDFILLANC;
                            'P_NRAUTOFORN'  => $af['NRAUTOFORN'], // adqTodasInconsAF.FieldByName('NRAUTOFORN').AsString;//P_NRAUTOFORN;
                            'P_CDGERENTE'   => $row['P_CDGERENTE'] // P_CDGERENTE;
                        );
                        $this->sup05100AnaliseAFService->acmUpAFIncons($paramsAcmUpAFIncons);
                    } else {
                        $stMsg = self::MENSAGENS['Erro_LibAF'];
                    }
                }
            }
        } else {
            try {
                $stNrListaAF = $this->prox_Lista_AF($row['P_AUTOFORNE']);
                
                $paramsAdqListaInconsAF = array(
                    'P_NRLISTAAF'  => $stNrListaAF
                );
                $listaInconsAF = $this->sup05100AnaliseAFService->adqListaInconsAF($paramsAdqListaInconsAF);
                
                foreach ($listaInconsAF as $af) {
                    $paramsAdqLimiteGerente = array(
                        'P_CDFILDEST'   => $af['CDFILDEST'], // adqListaInconsAF.FieldByName('CDFILDEST').AsString;//P_CDFILDEST;
                        'P_CDFILLANC'   => $af['CDFILLANC'], // adqListaInconsAF.FieldByName('CDFILLANC').AsString;//P_CDFILLANC;
                        'P_NRAUTOFORN'  => $af['NRAUTOFORN'], // adqListaInconsAF.FieldByName('NRAUTOFORN').AsString;//P_NRAUTOFORN;
                        'P_CDGERENTE'   => $row['P_CDGERENTE'], // P_CDGERENTE;
                        'P_CDOPERADOR'  => $af['CDOPERADOR'] // adqListaInconsAF.FieldByName('CDOPERADOR').AsString;
                    );
                    $limiteGerente = $this->sup05100AnaliseAFService->adqLimiteGerente($paramsAdqLimiteGerente);
                    
                    if(!empty($limiteGerente)) {
                        $paramsAcmUpAFIncons = array(
                            'P_CDFILDEST'   => $af['CDFILDEST'], // adqListaInconsAF.FieldByName('CDFILDEST').AsString;//P_CDFILDEST;
                            'P_CDFILLANC'   => $af['CDFILLANC'], // adqListaInconsAF.FieldByName('CDFILLANC').AsString;//P_CDFILLANC;
                            'P_NRAUTOFORN'  => $af['NRAUTOFORN'], // adqListaInconsAF.FieldByName('NRAUTOFORN').AsString;//P_NRAUTOFORN;
                            'P_CDGERENTE'   => $row['P_CDGERENTE'] // P_CDGERENTE;
                        );
                        $this->sup05100AnaliseAFService->acmUpAFIncons($paramsAcmUpAFIncons);
                    } else {
                        $stMsg = self::MENSAGENS['Erro_LibAF'];
                    }
                }
            } finally {
                $this->delete_Lista($stNrListaAF);
            }
        }
        
        return $stMsg;
    }
    
    public function realizaValidacoesLiberarAF(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        
        try {
            $this->validaCompradorLiberarAF($row);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function validaCompradorLiberarAF($row) {
        if (!empty($row['CDCOMPRADOR'])) {
            $vdComprador = $this->sup05100AnaliseAFService->qryVdComprador( array('P_CDOPERADOR' => $row['CDCOMPRADOR'] )); // ParamByName('P_CDOPERADOR').AsString := medCdComprador.Text;
            
            if (empty($vdComprador)) {
                throw new \Exception(self::MENSAGENS['Compr_NCAD']);
            }
        } else {
            throw new \Exception(self::MENSAGENS['Comprad_Inf']);
        }
    }
    
    /********************************************
     *           Gerente Comprador              *
     ********************************************/
     
    public function verificarGerenteComprador(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        
        try {
            $gerentes = $this->sup05100AnaliseAFService->getGerentePorComprador($row['CDCOMPRADOR']);
            
            if (empty($gerentes)) {
                // throw new \Exception('Não há gerente cadastrado para comprador.');
                throw new \Exception(self::MENSAGENS['GERENTE_COMP_N_CAD']);
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    public function validarSenhaGerente(DTO\Request\Row $request, DTO\Response $response) {
        try  {
            $row = $request->getRow();
            $dataSourceName = 'operador';
            $conditions = array();
            
            array_push($conditions, array('columnName' => 'CDOPERADOR', 'operator' => '=', 'value' => $row['CDGERENTE']));
            
            $dataSet = $this->crudService->getFilteredDataSet($dataSourceName, $conditions)->getRows();
            if (md5($row['PASSWORD']) != $dataSet[0]['CDSENHAOPERWEB']) {
                throw new \Exception(self::MENSAGENS['SENHA_INVALIDA'], 1);
            }
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function defineCookieEmail(DTO\Request $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            
            $isResend = isset($row['ISRESENDEMAIL']) ? $row['ISRESENDEMAIL'] : false;
            
            if (strpos($row['MENSAGEM'], 'data:image') == false) {
                $row['MENSAGEM'] = str_replace('src="', 'src="data:image', $row['MENSAGEM']);
            }
            
            if($isResend) {
                $this->cmpUtilities->defineCookieEmail(self::SUBJECT_RESEND_EMAIL_COOKIE_NAME, $row['ASSUNTO'], time() + (86400 * 999));
                $this->cmpUtilities->defineCookieEmail(self::RESEND_EMAIL_COOKIE_NAME, $row['MENSAGEM'], time() + (86400 * 999));
                $this->cmpUtilities->defineCookieEmail(self::RESEND_TO_OPERATOR, $row['enviarCopiaOperador'], time() + (86400 * 999));
            } else {
                $this->cmpUtilities->defineCookieEmail(self::SUBJECT_EMAIL_COOKIE_NAME, $row['ASSUNTO'], time() + (86400 * 999));
                $this->cmpUtilities->defineCookieEmail(self::EMAIL_COOKIE_NAME, $row['MENSAGEM'], time() + (86400 * 999));
                $this->cmpUtilities->defineCookieEmail(self::SEND_TO_OPERATOR, $row['enviarCopiaOperador'], time() + (86400 * 999));
            }
            
            $response->addMessage(new Message('Cookie successful'));
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function retornaCookieEmail(DTO\Request $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            
            $isResend = false;
            
            if(count($row)) {
                $isResend = isset($row['ISRESENDEMAIL']) ? $row['ISRESENDEMAIL'] : false;
            }
            
            if($isResend) {
                $assunto = $this->cmpUtilities->retornaCookieEmail(self::SUBJECT_RESEND_EMAIL_COOKIE_NAME);
                $mensagem = $this->cmpUtilities->retornaCookieEmail(self::RESEND_EMAIL_COOKIE_NAME);    
                $copyToOperator = $this->cmpUtilities->retornaCookieEmail(self::RESEND_TO_OPERATOR);
            } else {
                $assunto = $this->cmpUtilities->retornaCookieEmail(self::SUBJECT_EMAIL_COOKIE_NAME);
                $mensagem = $this->cmpUtilities->retornaCookieEmail(self::EMAIL_COOKIE_NAME);
                $copyToOperator = $this->cmpUtilities->retornaCookieEmail(self::SEND_TO_OPERATOR);
            }
            
            if(empty($assunto)) {
                $assunto = 'Autorização de Fornecimento';
            }
            if(empty($mensagem)) {
                $mensagem = self::DEFAULT_EMAIL_TEMPLATE . $this->environment->getCurrentUserName();
            }
            if(empty($copyToOperator)) {
                $copyToOperator = $this->sup05100AnaliseAFService->possuiAlcada() ? 'S' : 'N'; 
            }

            $response->addMessage(new Message($assunto));
            $response->addMessage(new Message($mensagem));
            $response->addMessage(new Message($copyToOperator));
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /********************************************
     *                Validações                *
     ********************************************/
    
    public function verificaOperadorRelacionado(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        
        try {
            self::isOperadorCompradorDaAf($row['CDCOMPRADOR'], $response);
            self::isOperadorRelacionado($row['CDCOMPRADOR'], $response);
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function isApuracaoEmAnalise($idsitapuprec) {
        if($idsitapuprec != '1') {
            throw new \Exception(self::MENSAGENS['ANALISE_AF_CONCLUIDA']);
        }
        
        return true;
    }
    
    public function isOperadorCompradorDaAf($cdcomprador, &$response = null) {
        if ($cdcomprador !== $this->environment->getCdoperador()) {
            return self::addMessage($response, [ 'OPERADORCOMPRADOR' => false ]);
        }
        
        return self::addMessage($response, [ 'OPERADORCOMPRADOR' => true ]);
    }
    
    public function isOperadorRelacionado($cdcomprador, &$response = null) {
        $operadorRelacionado = $this->sup05100AnaliseAFService->retornaOperadorRelacionado($cdcomprador);
        
        if (empty($operadorRelacionado)) {
            return self::addMessage($response, [ 'OPERADORRELACIONADO' => false ]);
        }
        
        $key = array_search($this->environment->getCdoperador(), array_column($operadorRelacionado, 'CDOPERADOR'));
        
        if ($key === false) {
            return self::addMessage($response, [ 'OPERADORRELACIONADO' => false ]);
            // throw new \Exception(self::MENSAGENS['Oper_N_Relac']);
        }
        
        self::isOperadorRelacionadoAutorizado($operadorRelacionado[$key]['IDAFCMPOPER'], $response);
        
        return self::addMessage($response, [ 'OPERADORRELACIONADO' => true ]);
        // return true;
    }
    
    public function isOperadorRelacionadoAutorizado($idafcmpoper, &$response = null) {
        if($idafcmpoper == 'N') {
            return self::addMessage($response, [ 'OPERADORAUTORIZADO' => false ]);
            // throw new \Exception(self::MENSAGENS['Oper_Relac_N_Aut']);
        }
        
        return self::addMessage($response, [ 'OPERADORAUTORIZADO' => true ]);
        // return true;
    }
    
    private function getConditionsItensRemanejamento($filterCriteria)
    {
        $conditions = array();
        array_push($conditions, 
            array('columnName' => 'P_CDFILDEST',   'operator' => '=', 'value' => self::retornaValorParamFilterCriteria($filterCriteria, 'P_CDFILDEST')),
            array('columnName' => 'P_CDFILLANC',   'operator' => '=', 'value' => self::retornaValorParamFilterCriteria($filterCriteria, 'P_CDFILLANC')),
            array('columnName' => 'P_NRSEQAPURAF', 'operator' => '=', 'value' => self::retornaValorParamFilterCriteria($filterCriteria, 'P_NRSEQAPURAF')),
            array('columnName' => 'P_CDPRODUTO',   'operator' => '=', 'value' => self::retornaValorParamFilterCriteria($filterCriteria, 'P_CDPRODUTO')),
            array('columnName' => 'P_NRAUTOFORN',  'operator' => '=', 'value' => self::retornaValorParamFilterCriteria($filterCriteria, 'P_NRAUTOFORN'))
        );
        
        return $conditions;
    }
    
    public function getItensApuracao(DTO\Request\Filter $request, DTO\Response $response)
    {
        try {
            $filterCriteria = $request->getFilterCriteria();
            $conditions = $filterCriteria->getConditions();
    
            $dataSourceName = 'sup05100_itensRemanejamento';
            $dataSet = $this->crudService->getFilteredDataSet($dataSourceName, self::getConditionsItensRemanejamento($filterCriteria))->getRows();
            
            $parents = array();
            $dataSetApura = array();
    
            foreach ($dataSet as &$itens) {
                $nrId = $itens['CDFILLANC'].$itens['CDFILDEST'].$itens['NRAUTOFORN'].$itens['CDPRODUTO'].self::formatDateforId($itens['DTPROGENTAF']);
                if (!isset($parents[$nrId])) {
                    $parents[$nrId] = array(
                        'ID'            => $nrId,
                        'CDFILDEST'     => $itens['CDFILDEST'],
                        'NMFILIAL'      => $itens['NMFILIAL'],
                        'IDFUNCAOFILI'  => $itens['IDFUNCAOFILI'],
                        'CDFILCENTCMP'  => $itens['CDFILCENTCMP'],
                        'CDFILLANC'     => $itens['CDFILLANC'],
                        'NRAUTOFORN'    => $itens['NRAUTOFORN'],
                        'DTPROGENTAF'   => $itens['DTPROGENTAF'],
                        'DTENTRITMSOL'  => $itens['DTENTRITMSOL'],
                        'CDPRODUTO'     => $itens['CDPRODUTO'],
                        'CDARVPROD'     => $itens['CDARVPROD'],
                        'NMPRODUTO'     => $itens['NMPRODUTO'],
                        'CDPRODESTO'    => $itens['CDPRODESTO'],
                        'VRFATOCONV'    => $itens['VRFATOCONV'],
                        'CDMARCA'       => $itens['CDMARCA'],
                        'QTPROGENTAF'   => $itens['QTPROGENTAF'],
                        'VRPREUNIAF'    => $itens['VRPREUNIAF'],
                        'IDBAIXAAF'     => $itens['IDBAIXAAF'],
                        'NRSEQAPURAF'   => $itens['NRSEQAPURAF'],
                        'CDOPERADOR'    => $itens['CDOPERADOR'],
                        'DTAUTOFORN'    => $itens['DTAUTOFORN'],
                        'DSINCONSITAF'  => $itens['DSINCONSITAF'],
                        'IDMOTPRESCAF'  => $itens['IDMOTPRESCAF'],
                        'IDTIPOLANITM'  => $itens['IDTIPOLANITM'],
                        'NRCOTPREITAF'  => $itens['NRCOTPREITAF'],
                        'NRLISTAITAF'   => $itens['NRLISTAITAF'],
                        'CDOPERADOR'    => $itens['CDOPERADOR']
                    );
                }
                
                $itens['PARENT'] = $parents[$nrId]['ID'];
                
                $relacAF = self::retornaValorParamFilterCriteria($filterCriteria, 'P_ITENSRELAC');
                
                if($relacAF){
                    $dataSourceName = 'sup05100_itensApuRelac';
                } else {
                    $dataSourceName = 'sup05100_itensApuracao';
                }
                
    
                $conditionsApu = array();
                array_push($conditionsApu, 
                    array('columnName' => 'P_PARENT',       'operator' => '=', 'value' => $itens['PARENT']),
                    array('columnName' => 'P_CDPRODUTO',    'operator' => '=', 'value' => $itens['CDPRODUTO']),
                    array('columnName' => 'P_QTPROGENTAF',  'operator' => '=', 'value' => $itens['QTPROGENTAF']),
                    array('columnName' => 'P_NRSEQITMAP',   'operator' => '=', 'value' => $itens['NRSEQAPURAF']),
                    array('columnName' => 'P_CDFILDEST',    'operator' => '=', 'value' => $itens['CDFILDEST']),
                    array('columnName' => 'P_CDFILLANC',    'operator' => '=', 'value' => $itens['CDFILLANC']),
                    array('columnName' => 'P_CDFORNECED',   'operator' => '=', 'value' => $itens['CDFORNECED']),
                    array('columnName' => 'P_DTENTRAPUT',   'operator' => '=', 'value' => $itens['DTENTRITMSOL']),
                    array('columnName' => 'P_CDMARCA',      'operator' => '=', 'value' => $itens['CDMARCA'])
                );
                
                $dataSetApura = array_merge($dataSetApura, $this->crudService->getFilteredDataSet($dataSourceName, $conditionsApu)->getRows());
            };
            
            $newParents = array_values($parents);
            $child = array_values($dataSetApura);
            $returnDataset = array_merge($newParents, $child);
            
            $response->addDataSet(new DataSet("itensApuracao", $returnDataset));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    private function verifyExisteAF($row)
    {
        $dataSourceName = 'sup05100_existeAFForn';
        $conditions = array();
        
        array_push($conditions, 
            array('columnName' => 'P_CDFILDEST',     'operator' => '=', 'value' => $row['_parent']['CDFILDEST']),
            array('columnName' => 'P_CDFILLANC',     'operator' => '=', 'value' => $row['_parent']['CDFILLANC']),
            array('columnName' => 'P_DTPROGENTAF',   'operator' => '=', 'value' => $row['_parent']['DTPROGENTAF']),
            array('columnName' => 'P_CDFORNECED',    'operator' => '=', 'value' => $row['CDFORNITMAP']),
            array('columnName' => 'P_NRSEQAPURPREC', 'operator' => '=', 'value' => $row['_parent']['NRSEQAPURAF']),
            array('columnName' => 'NRORG',           'operator' => '=', 'value' => $this->environment->getNrOrg())
        );

        $result = $this->crudService->getFilteredDataSet($dataSourceName, $conditions)->getRows();
        return is_array($result) && count($result) ? $result[0] : false;
    }
    
    private function verifyExisteItemAF($row, $nrAF)
    {
        $dataSourceName = 'sup05100_existeItemAF';
        $conditions = array();
        
        array_push($conditions, 
            array('columnName' => 'P_CDFILDEST',     'operator' => '=', 'value' => $row['_parent']['CDFILDEST']),
            array('columnName' => 'P_CDFILLANC',     'operator' => '=', 'value' => $row['_parent']['CDFILLANC']),
            array('columnName' => 'P_DTPROGENTAF',   'operator' => '=', 'value' => $row['_parent']['DTPROGENTAF']),
            array('columnName' => 'P_NRSEQAPURPREC', 'operator' => '=', 'value' => $row['_parent']['NRSEQAPURAF']),
            array('columnName' => 'P_CDPRODUTO',     'operator' => '=', 'value' => $row['CDPRODUTO']),
            array('columnName' => 'P_CDMARCA',       'operator' => '=', 'value' => $row['CDMARCITMAP']),
            array('columnName' => 'P_CDFORNECED',    'operator' => '=', 'value' => $row['CDFORNITMAP']),
            array('columnName' => 'P_NRAUTOFORN',    'operator' => '=', 'value' => $nrAF),
            array('columnName' => 'NRORG',           'operator' => '=', 'value' => $this->environment->getNrOrg())
        );
        
        $result = $this->crudService->getFilteredDataSet($dataSourceName, $conditions)->getRows();
        return is_array($result) && count($result) ? $result[0] : false;
    }
    
    private function insertMovOpAF($row, $nrautoforne, $cdmarca, $operacao)
    {
        $dsAlterAF = $operacao === 'E' ? 'Análise de AF-Exclusão do Item.' : $operacao === 'A' ? 'Análise de AF-Atualização do Item' : 'Análise de AF-Inclusão do Item';
        $dsJustOpe = $operacao === 'E' ? 'Análise de AF-Exclusão do Item/Remanej.' : $operacao === 'A' ? 'Análise de AF-Atual. Item/Remanej.' : 'Análise de AF-Incl. do Item/Remanej.';

        $params = array(
            'DTPROGENTAF' => self::formataDataString($row['_parent']['DTPROGENTAF']),
            'DSALTERAF' => $dsAlterAF,
            'DSJUSTOPERMOV' => $dsJustOpe,
            'IDOPERAMOVAF' => $operacao,
            'IDUTATUALIZASOL' => 'N',
            'CDFILDEST' => $row['_parent']['CDFILDEST'],
            'CDFILLANC' => $row['_parent']['CDFILLANC'],
            'NRAUTOFORN' => $nrautoforne,
            'CDPRODUTO' => $row['CDPRODUTO'],
            'CDMARCA' => $cdmarca
        );
                                                                    
        $this->sup05100AnaliseAFService->insereMovopaf($params);
    }
    
    private function setArrayFechamento($row, $nrautoforn)
    {
        $dataset = array(
            'CDFILDEST'    => $row['_parent']['CDFILDEST'],
            'CDFILLANC'    => $row['_parent']['CDFILLANC'],
            'CDFILCENTCMP' => $row['_parent']['CDFILCENTCMP'],
            'NRAUTOFORN'   => $nrautoforn,
            'CDFILIPROC'   => $this->environment->getCdFilial(),
            'CDFORNECED'   => $row['CDFORNITMAP'],
            'IDTPFORPGTFO' => $row['IDTPFORPGTFO'] ? $row['IDTPFORPGTFO'] : 'X'
        );
        
        return $dataset;
    }
    
    private function insertFormPgtoAF($row)
    {
        if($row['IDTPFORPGTFO'] === 'V' && $this->sup05100AnaliseAFService->retornaAutoforne($row['CDFILDEST'], $row['CDFILLANC'], $row['NRAUTOFORN'])) {
            
            $conditions = array();
            array_push($conditions, 
                array('columnName' => 'CDFORNECED', 'operator' => '=', 'value' => $row['CDFORNECED'])
            );
            
            $prazos = $this->crudService->getFilteredDataSet('prazopgtforn', $conditions)->getRows();
            return $this->sup05100AnaliseAFService->insertPrazoPgtAF($row, $prazos); 
        }
    }
    
    public function concluiRemanejamentoApuracao(DTO\Request\Row $request, DTO\Response $response)
    {
        $datasetRow = $request->getRow();
        $fechamentoAF = array();
   
        try {
            foreach ($datasetRow as $row) {
                $stNrAutoForn = '';

                $this->sup05100AnaliseAFService->iniciaConexao();
                
                if($row['IDGERAFDTENT'] === 'S') {
                    $existeAF = self::verifyExisteAF($row);
                    
                    if(is_array($existeAF) && count($existeAF)) {
                    
                        $existeItemAF = self::verifyExisteItemAF($row, $existeAF['NRAUTOFORN']);
                        
                        if(is_array($existeItemAF) && count($existeItemAF)) {
                            //********IF NÃO VALIDADO***********
                            $params = array(
                                "QTPROGENTAF" => $existeItemAF['QTPROGENTAF'] + $row['QTPRODORIG'],
                                "CDFILDEST" => $row['_parent']['CDFILDEST'],
                                "CDFILLANC" => $row['_parent']['CDFILLANC'],
                                "NRAUTOFORN" => $existeAF['NRAUTOFORN'],
                                "CDPRODUTO" => $existeItemAF['CDPRODUTO'],
                                "CDMARCA" => $existeItemAF['CDMARCA'],
                                "DTPROGENTAF" => $existeItemAF['DTPROGENTAF']
                            );
                            
                            $this->sup05100AnaliseAFService->atualizaQtItemAutfRemanejo($params);
                            self::insertMovOpAF($row, $existeAF['NRAUTOFORN'], $row['CDMARCITMAP'], 'A');
                            
                        } else {
                            $params = array(
                                'CDFILDEST' => $row['_parent']['CDFILDEST'],
                                'CDFILLANC' => $row['_parent']['CDFILLANC'],
                                'NRAUTOFORNDEST' => $existeAF['NRAUTOFORN'],
                                'CDMARCADEST' => $row['CDMARCITMAP'],
                                'CDPRODUTO' => $row['CDPRODUTO'],
                                'DTPROGENTAFDEST' => self::formataDataString($row['_parent']['DTPROGENTAF']),
                                'VRPREUNIAF' => $row['VRPRECUNIAP'],
                                'DTENTRITMSOL' => null,
                                'NRAUTOFORNORIG' => $row['_parent']['NRAUTOFORN'],
                                'DTPROGENTAFORIG' => self::formataDataString($row['_parent']['DTPROGENTAF']), 
                                'CDPRODORIG' => $row['_parent']['CDPRODUTO'], 
                                'CDMARCA' => $row['_parent']['CDMARCA'],
                                'QTPROGENTAF' => $row['QTPRODORIG'],
                                'VRPERDESAF' => $row['VRPERDESIAP']
                            );
                                                                                            
                            $this->sup05100AnaliseAFService->insereItemAutfRemanejo($params);
                            self::insertMovOpAF($row, $existeAF['NRAUTOFORN'], $row['CDMARCITMAP'], 'I');
                        }
                    
                        //Alteração para melhoria de performance chamando a FechamentoAF no fim do processo
                        array_push($fechamentoAF, self::setArrayFechamento($row, $existeAF['NRAUTOFORN']));
                        array_push($fechamentoAF, self::setArrayFechamento($row, $row['_parent']['NRAUTOFORN']));

                    } else {
                        if(empty($stNrAutoForn)) {
                            $stNrAutoForn = self::generateCode('AUTOFORNE');
                            
                            $params = array(
                                'CDFILDEST' => $row['_parent']['CDFILDEST'],
                                'CDFILLANC' => $row['_parent']['CDFILLANC'],
                                'NRAUTOFORN' => $stNrAutoForn,
                                'CDFILCENTCMP' => $row['_parent']['CDFILCENTCMP'],
                                'IDAFOPERLOG' => null,
                                'CDFILIOPELOG' => null,
                                'CDPRODUTO' => $row['CDPRODUTO'],
                                'NRSEQAPURPREC' => $row['_parent']['NRSEQAPURAF'],
                                'CDFORNECED' => $row['CDFORNITMAP']
                            );
                            
                            $this->sup05100AnaliseAFService->insertAf($params);
                        }
                         
                        $params = array(
                            'CDFILDEST' => $row['_parent']['CDFILDEST'],
                            'CDFILLANC' => $row['_parent']['CDFILLANC'],
                            'NRAUTOFORNDEST' => $stNrAutoForn,
                            'CDMARCADEST' => $row['CDMARCITMAP'],
                            'CDPRODUTO' => $row['CDPRODUTO'],
                            'DTPROGENTAFDEST' => self::formataDataString($row['_parent']['DTPROGENTAF']),
                            'VRPREUNIAF' => $row['VRPRECUNIAP'],
                            'DTENTRITMSOL' => null,
                            'NRAUTOFORNORIG' => $row['_parent']['NRAUTOFORN'],
                            'DTPROGENTAFORIG' => self::formataDataString($row['_parent']['DTPROGENTAF']), 
                            'CDPRODORIG' => $row['_parent']['CDPRODUTO'], 
                            'CDMARCA' => $row['_parent']['CDMARCA'],
                            'QTPROGENTAF' => $row['QTPRODORIG'],
                            'VRPERDESAF' => $row['VRPERDESIAP']
                        );
                                                                                         
                        $this->sup05100AnaliseAFService->insereItemAutfRemanejo($params);
                        self::insertMovOpAF($row, $stNrAutoForn, $row['CDMARCITMAP'], 'I');
                        
                          
                        //Alteração para melhoria de performance chamando a FechamentoAF no fim do processo
                        array_push($fechamentoAF, self::setArrayFechamento($row, $stNrAutoForn));
                        array_push($fechamentoAF, self::setArrayFechamento($row, $row['_parent']['NRAUTOFORN']));
                    }
                } else {
                    $existeAF = self::verifyExisteAF($row);
                      
                    if(is_array($existeAF) && count($existeAF)) {
                        $existeItemAF = self::verifyExisteItemAF($row, $existeAF['NRAUTOFORN']);
                                
                        if(is_array($existeItemAF) && count($existeItemAF)) {
                            //********IF NÃO VALIDADO***********
                            $params = array(
                                "QTPROGENTAF" => $existeItemAF['QTPROGENTAF'] + $row['QTPRODORIG'],
                                "CDFILDEST" => $row['_parent']['CDFILDEST'],
                                "CDFILLANC" => $row['_parent']['CDFILLANC'],
                                "NRAUTOFORN" => $existeAF['NRAUTOFORN'],
                                "CDPRODUTO" => $existeItemAF['CDPRODUTO'],
                                "CDMARCA" => $existeItemAF['CDMARCA'],
                                "DTPROGENTAF" => $existeItemAF['DTPROGENTAF']
                            );
                            
                            $this->sup05100AnaliseAFService->atualizaQtItemAutfRemanejo($params);
                            self::insertMovOpAF($row, $existeAF['NRAUTOFORN'], $row['CDMARCITMAP'], 'A');

                        } else {
                            $params = array(
                                'CDFILDEST' => $row['_parent']['CDFILDEST'],
                                'CDFILLANC' => $row['_parent']['CDFILLANC'],
                                'NRAUTOFORNDEST' => $existeAF['NRAUTOFORN'],
                                'CDMARCADEST' => $row['CDMARCITMAP'],
                                'CDPRODUTO' => $row['CDPRODUTO'],
                                'DTPROGENTAFDEST' => self::formataDataString($row['_parent']['DTPROGENTAF']),
                                'VRPREUNIAF' => $row['VRPRECUNIAP'],
                                'DTENTRITMSOL' => null,
                                'NRAUTOFORNORIG' => $row['_parent']['NRAUTOFORN'],
                                'DTPROGENTAFORIG' => self::formataDataString($row['_parent']['DTPROGENTAF']), 
                                'CDPRODORIG' => $row['_parent']['CDPRODUTO'], 
                                'CDMARCA' => $row['_parent']['CDMARCA'],
                                'QTPROGENTAF' => $row['QTPRODORIG'],
                                'VRPERDESAF' => $row['VRPERDESIAP']
                            );
                                                                                            
                            $this->sup05100AnaliseAFService->insereItemAutfRemanejo($params);
                            self::insertMovOpAF($row, $existeAF['NRAUTOFORN'], $row['CDMARCITMAP'], 'I');
                            
                        }
                    } else {
                        if(empty($stNrAutoForn)) {
                            $stNrAutoForn = self::generateCode('AUTOFORNE');
                            
                            $params = array(
                                'CDFILDEST' => $row['_parent']['CDFILDEST'],
                                'CDFILLANC' => $row['_parent']['CDFILLANC'],
                                'NRAUTOFORN' => $stNrAutoForn,
                                'CDFILCENTCMP' => $row['_parent']['CDFILCENTCMP'],
                                'IDAFOPERLOG' => null,
                                'CDFILIOPELOG' => null,
                                'CDPRODUTO' => $row['CDPRODUTO'],
                                'NRSEQAPURPREC' => $row['_parent']['NRSEQAPURAF'],
                                'CDFORNECED' => $row['CDFORNITMAP']
                            );
                            
                            $this->sup05100AnaliseAFService->insertAf($params);
                        }
                     
                        $params = array(
                            'CDFILDEST' => $row['_parent']['CDFILDEST'],
                            'CDFILLANC' => $row['_parent']['CDFILLANC'],
                            'NRAUTOFORNDEST' => $stNrAutoForn,
                            'CDMARCADEST' => $row['CDMARCITMAP'],
                            'CDPRODUTO' => $row['CDPRODUTO'],
                            'DTPROGENTAFDEST' => null,
                            'DTENTRITMSOL' => null,
                            'VRPREUNIAF' => $row['VRPRECUNIAP'],
                            'NRAUTOFORNORIG' => $row['_parent']['NRAUTOFORN'],
                            'DTPROGENTAFORIG' => self::formataDataString($row['_parent']['DTPROGENTAF']), 
                            'CDPRODORIG' => $row['_parent']['CDPRODUTO'], 
                            'CDMARCA' => $row['_parent']['CDMARCA'],
                            'QTPROGENTAF' => $row['QTPRODORIG'],
                            'VRPERDESAF' => $row['VRPERDESIAP']
                        );
                                                                                         
                        $this->sup05100AnaliseAFService->insereItemAutfRemanejo($params);
                        self::insertMovOpAF($row, $stNrAutoForn, $row['CDMARCITMAP'], 'I');
                     
                    }
                }
                
               
                $this->contaPagar->excluiNFContaPagar($row['_parent']['CDFILDEST'], 
                                                      $row['_parent']['CDFILLANC'], 
                                                      $row['_parent']['NRAUTOFORN'], false);
                
                $this->sup05100AnaliseAFService->deletaItemAutf($row['_parent']['CDFILDEST'], 
                                                                $row['_parent']['CDFILLANC'], 
                                                                $row['_parent']['NRAUTOFORN'], 
                                                                $row['_parent']['CDPRODUTO'], 
                                                                self::formataDataString($row['_parent']['DTPROGENTAF']), 
                                                                $row['_parent']['CDMARCA']);
                
                $this->sup05100AnaliseAFService->deletaAutoforne($row['_parent']['CDFILDEST'], 
                                                                 $row['_parent']['CDFILLANC'], 
                                                                 $row['_parent']['NRAUTOFORN']); 
               
                self::insertMovOpAF($row, $row['_parent']['NRAUTOFORN'], $row['_parent']['CDMARCA'], 'E');
                
                $params = array(
                    "IDITMSELAPUR"  => 'N',
                    "P_CDCENTITMAP" => $row['_parent']['CDFILCENTCMP'],
                    "P_NRSEQITMAP"  => $row['_parent']['NRSEQAPURAF'],
                    "P_CDREGITMAP"  => $row['CDREGITMAP'],
                    "P_CDFORNITMAP" => null,
                    "P_NRCOLEITMAP" => $row['NRCOLEITMAP'],
                    "P_DTENTRAPUT"  => $row['_parent']['DTENTRITMSOL'],
                    "P_CDFILIAP"    => $row['_parent']['CDFILDEST'],
                    "P_CDPRODITMAP" => $row['_parent']['CDPRODUTO'],
                    "P_CDMARCITMAP" => $row['_parent']['CDMARCA']
                );
                        
                $this->sup05100AnaliseAFService->updateItmApur($params);
                
                $params = array(
                    "IDITMSELAPUR"  => 'S',
                    "P_CDCENTITMAP" => $row['_parent']['CDFILCENTCMP'],
                    "P_NRSEQITMAP"  => $row['_parent']['NRSEQAPURAF'],
                    "P_CDREGITMAP"  => $row['CDREGITMAP'],
                    "P_CDFORNITMAP" => $row['CDFORNITMAP'],
                    "P_NRCOLEITMAP" => $row['NRCOLEITMAP'],
                    "P_DTENTRAPUT"  => $row['_parent']['DTENTRITMSOL'],
                    "P_CDFILIAP"    => $row['_parent']['CDFILDEST'],
                    "P_CDPRODITMAP" => $row['CDPRODUTO'],
                    "P_CDMARCITMAP" => $row['CDMARCITMAP']
                );
                        
                $this->sup05100AnaliseAFService->updateItmApur($params);
               
                //Alteração para melhoria de performance chamando a FechamentoAF no fim do processo
                array_push($fechamentoAF, self::setArrayFechamento($row, $stNrAutoForn));
                array_push($fechamentoAF, self::setArrayFechamento($row, $row['_parent']['NRAUTOFORN']));
                array_push($fechamentoAF, self::setArrayFechamento($row, $existeAF['NRAUTOFORN']));
            }
         
            foreach($fechamentoAF as $row) {
                self::insertFormPgtoAF($row);
                $this->fechamentoAF->fechaAF($row['CDFILDEST'], $row['CDFILLANC'], $row['NRAUTOFORN'], $row['CDFILCENTCMP']);
            }
            
            $this->sup05100AnaliseAFService->realizaFlush();
            $this->sup05100AnaliseAFService->realizaCommit();

        } catch (\Exception $e) {
            $this->sup05100AnaliseAFService->realizaRollback();
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    private function formatDateforId($data) 
    {
        if(gettype($data) === 'string') {
            $data = substr($data, 0, 10);
            $data = str_replace('/', '-', $data);
            $data = new \DateTime($data);
            $data = $data->format('ymd');
        } else if($data instanceof \DateTime) {
            $data = $data->format('ymd');
        }
        
        return $data;
    }
    
    /****************************** Atualização da Solicitação de Compras ******************************/
    
    private function getRelacProdol($row)
    {
        $dataSourceName = 'sup05100_relacprodol';
        $conditions = array(
            ["columnName" => "P_CDPRODUTO", "operator" => "=", "value" => $row['CDPRODUTO']]
        );
        return $this->crudService->getFilteredDataSet($dataSourceName, $conditions)->getRows();
    }

    private function getItemsolirelacAF($row)
    {
        $conditions = array();
        array_push($conditions, 
            array('columnName' => 'P_CDFILDEST',    'operator' => '=', 'value' => $row['CDFILDEST']),
            array('columnName' => 'P_CDFILLANC',    'operator' => '=', 'value' => $row['CDFILLANC']),
            array('columnName' => 'P_NRAUTOFORN',   'operator' => '=', 'value' => $row['NRAUTOFORN']),
            array('columnName' => 'P_CDPRODUTO',    'operator' => '=', 'value' => $row['CDPRODUTO']),
            array('columnName' => 'P_CDMARCA',      'operator' => '=', 'value' => $row['CDMARCA']),
            array('columnName' => 'P_DTPROGENTAF',  'operator' => '=', 'value' => $row['DTPROGENTAF']),
            array('columnName' => 'P_NRSEQAPURAF',  'operator' => '=', 'value' => $row['NRAPURPREC']),
            array('columnName' => 'P_CDFILCENTCMP', 'operator' => '=', 'value' => $row['CDFILCENTCMP']),
            array('columnName' => 'NRORG',          'operator' => '=', 'value' => $this->environment->getNrOrg())
        );
    
        $dataSourceName = 'count_itemsolirelacaf';
        $itemSoliRelacAF = $this->crudService->getFilteredDataSet($dataSourceName, $conditions)->getRows();
        return $itemSoliRelacAF[0];
    }

    private function getAtuEntrItmSol($row, $itemsolirelacAF)
    {
        $conditions = array();
        array_push($conditions, 
            array('columnName' => 'P_CDFILDEST',    'operator' => '=', 'value' => $row['CDFILDEST']),
            array('columnName' => 'P_CDFILLANC',    'operator' => '=', 'value' => $row['CDFILLANC']),
            array('columnName' => 'P_NRAUTOFORN',   'operator' => '=', 'value' => $row['NRAUTOFORN']),
            array('columnName' => 'P_CDPRODUTO',    'operator' => '=', 'value' => $row['CDPRODUTO']),
            array('columnName' => 'P_CDMARCA',      'operator' => '=', 'value' => $row['CDMARCA']),
            array('columnName' => 'P_DTPROGENTAF',  'operator' => '=', 'value' => $row['DTPROGENTAF']),
            array('columnName' => 'NRORG',          'operator' => '=', 'value' => $this->environment->getNrOrg())
        );
        
        if($itemsolirelacAF['CONTADOR'] > 0) {
            array_push($conditions,
                array('columnName' => 'P_NRSEQAPURAF',  'operator' => '=', 'value' => $row['NRAPURPREC']),
                array('columnName' => 'P_CDFILCENTCMP', 'operator' => '=', 'value' => $row['CDFILCENTCMP'])
            );
            $dataSourceName = 'sup05100_atuentritm_apu_sol';
        } else {
            $dataSourceName = 'sup05100_atuentritm_sol';
        }
    
        return $this->crudService->getFilteredDataSet($dataSourceName, $conditions)->getRows();
    }

    private function getAtuItemSoli($row, $itemsolirelacAF, $stProduto, $stMarca)
    {
        $atuEntrItmSol = $this->getAtuEntrItmSol($row, $itemsolirelacAF);
        
        if(is_array($atuEntrItmSol) && count($atuEntrItmSol)) {
            $dtProgEntr = empty($atuEntrItmSol[0]['DTENTRITMSOL']) ? $row['DTPROGENTAF_OLD'] : $atuEntrItmSol[0]['DTENTRITMSOL'];
        } else {
            $dtProgEntr = $row['DTPROGENTAF_OLD'];
        }
        
        $conditions = array();
        array_push($conditions, 
            array('columnName' => 'P_CDFILSOLI',    'operator' => '=', 'value' => $row['CDFILDEST']),
            array('columnName' => 'P_CDPRODUTO',    'operator' => '=', 'value' => $stProduto),
            array('columnName' => 'P_CDMARCA',      'operator' => '=', 'value' => $stMarca),
            array('columnName' => 'P_DTPROGENTR',   'operator' => '=', 'value' => $dtProgEntr),
            array('columnName' => 'P_EXTRA',        'operator' => '=', 'value' => $row['IDCONSOLAPUEFET']),
            array('columnName' => 'P_CDPRODUTOOL',  'operator' => '=', 'value' => $row['CDPRODUTO']),
            array('columnName' => 'NRORG',          'operator' => '=', 'value' => $this->environment->getNrOrg())
        );
        
        if($itemsolirelacAF['CONTADOR'] > 0) {
            array_push($conditions,
                array('columnName' => 'P_NRSEQAPURAF',  'operator' => '=', 'value' => $row['NRAPURPREC']),
                array('columnName' => 'P_CDFILCENTCMP', 'operator' => '=', 'value' => $row['CDFILCENTCMP'])
            );
        }
        
        if($row['IDFUNCAOFILI'] === 'O') {
            if($itemsolirelacAF['CONTADOR'] > 0) {
                $dataSourceName = 'atuitemsoli_ol_apu';
            } else {
                $dataSourceName = 'atuitemsoli_ol';
            }
        } else {
            if($itemsolirelacAF['CONTADOR'] > 0) {
                $dataSourceName = 'atuitemsoli_apu';
            } else {
                $dataSourceName = 'atuitemsoli';
            }
        }
        
        return $this->crudService->getFilteredDataSet($dataSourceName, $conditions)->getRows();
    }
    
    private function createArrayParamsInsertMovOoSol($itemsoli, $qtdepoper, $operation) {
        return array(
            'CDFILSOLI'   => $itemsoli['CDFILSOLI'],
            'CDFILLAN'    => $itemsoli['CDFILLAN'],
            'NRSOLICMP'   => $itemsoli['NRSOLICMP'],
            'CDPRODUTO'   => $itemsoli['CDPRODUTO'],
            'DTPROGENTR'  => $itemsoli['DTPROGENTR'],
            'QTSOLIPROD'  => $itemsoli['QTSOLIPROD'],
            'QTDEPOPER'   => $qtdepoper,
            'DSJUSTOPER'  => self::ALTER_QTDE_ANALISE,
            'IDOPERAMOVS' => $operation
        );
    }
    
    private function getExistSolicita($row)
    {
        $conditions = array();
        array_push($conditions, 
            array('columnName' => 'P_CDFILSOLI',  'operator' => '=', 'value' => $row['CDFILSOLI']),
            array('columnName' => 'P_CDPRODUTO',  'operator' => '=', 'value' => $row['CDPRODUTO']),
            array('columnName' => 'P_DTPROGENTR', 'operator' => '=', 'value' => $row['DTPROGENTR']),
            array('columnName' => 'NRORG',        'operator' => '=', 'value' => $this->environment->getNrOrg())
        );
    
        $dataSourceName = 'sup05100_exist_solicita';
        return $this->crudService->getFilteredDataSet($dataSourceName, $conditions)->getRows();
    }
    
    private function updateItemSoli($row)
    {
        $boDelete = false;
        $doQuantDife = 0;
    
        if($row['IDFUNCAOFILI'] === 'O') {
            $relacProdOl = $this->getRelacProdol($row);
            $stProduto   = $relacProdOl['CDPRARVORIG'];
            $stMarca     = $relacProdOl['CDMARCA'];
            $stProdutoOl = $relacProdOl['CDPRARVORIG'];
            $doQuantDife = ($row['QTPROGENTAF'] * $relacProdOl['VRFATOCONV']) - ($row['QTPROGENTAF_OLD'] * $relacProdOl['VRFATOCONV']);
        } else {
            $stProduto   = $row['CDPRODUTO'];
            $stMarca     = $row['CDMARCA'];
            $stProdutoOl = '';
            $doQuantDife = ($row['QTPROGENTAF'] * $row['VRFATOCONV']) - ($row['QTPROGENTAF_OLD'] * $row['VRFATOCONV']);
        }

        if($row['QTPROGENTAF'] > 0) {
            if($row['ATUSOLICITACMP']) {
                if($doQuantDife !== 0) {
                    $itemsolirelacAF = $this->getItemsolirelacAF($row);
                    $atuItemSoli = $this->getAtuItemSoli($row, $itemsolirelacAF, $stProduto, $stMarca);
                    
                    foreach($atuItemSoli as $itemsoli) {
                        if($doQuantDife !== 0) {
                            if($doQuantDife > 0) {
                                $this->sup05100AnaliseAFService->updateQtSoliProd($itemsoli, $doQuantDife);
                                $this->sup05100AnaliseAFService->updateQtSoliProdApu($row, $itemsoli, $doQuantDife);
                                $qtdepoper = $itemsoli['QTSOLIPROD'] + $doQuantDife/$itemsoli['VRFATOSOLI'];
                                $this->sup05100AnaliseAFService->insertMovOpSol($this->createArrayParamsInsertMovOoSol($itemsoli, $qtdepoper, 'A'));
                                
                                $doQuantDife = 0;
                            } else {
                                if($itemsoli['QTSOLIPROD'] > abs($doQuantDife / $itemsoli['VRFATOSOLI'])) {
                                    $this->sup05100AnaliseAFService->updateQtSoliProd($itemsoli, $doQuantDife);
                                    $this->sup05100AnaliseAFService->updateQtSoliProdApu($row, $itemsoli, $doQuantDife);
                                    $qtdepoper = $itemsoli['QTSOLIPROD'] + $doQuantDife/$itemsoli['VRFATOSOLI'];
                                    $this->sup05100AnaliseAFService->insertMovOpSol($this->createArrayParamsInsertMovOoSol($itemsoli, $qtdepoper, 'A'));
                                } else {
                                    $existSolicita = $this->getExistSolicita($itemsoli);
                                
                                    if($existSolicita[0]['CONTADOR'] > 1) {
                                        $this->sup05100AnaliseAFService->deleteItemSoli($itemsoli);
                                        $this->sup05100AnaliseAFService->insertMovOpSol($this->createArrayParamsInsertMovOoSol($itemsoli, 0, 'E'));
                                    }
                                }
                                $doQuantDife = $doQuantDife + ($itemsoli['QTSOLIPROD'] * $itemsoli['VRFATOSOLI']);
                                
                                if($doQuantDife > 0) {
                                    $doQuantDife = 0;
                                }
                            }
                        }
                    }
                }
            }
        } else {
            $boDelete = true;
        }
        

        if(self::formataDataString($row['DTPROGENTAF']) !== self::formataDataString($row['DTPROGENTAF_OLD'])) {
            $itemsolirelacAF = $this->getItemsolirelacAF($row);
            if($itemsolirelacAF['CONTADOR'] > 0) {
                $this->sup05100AnaliseAFService->updateItemSoliRelacAF($row);
            }            
        }
        
        
        if($boDelete) {
            if($row['ATUSOLICITACMP']) {
                $itemsolirelacAF = $this->getItemsolirelacAF($row);
                $atuItemSoli = $this->getAtuItemSoli($row, $itemsolirelacAF, $stProduto, $stMarca);
                
                foreach($atuItemSoli as $itemsoli) {
                    $this->sup05100AnaliseAFService->deleteItemSoli($itemsoli);
                    $this->sup05100AnaliseAFService->insertMovOpSol($this->createArrayParamsInsertMovOoSol($itemsoli, 0, 'E'));
                    
                    if ($itemsolirelacAF['CONTADOR'] === 0) {
                        break;
                    }
                }
            }
        }
    }   
    
    /****************************** Fim - Atualização da Solicitação de Compras ******************************/
    
    public function resendEmailandSaveFile(DTO\Request\Row $request, DTO\Response $response)
    {
        ini_set('max_execution_time', 9999);
        try {
            $row = $request->getRow();
            
            
            $afsApuracao = $row['ALL_DATA_APURACAO'];
            $currentRow = $row['CURRENT_ROW']; 
            
            $listaCdforneced = array();
        
            if(isset($currentRow['NMFANTFORN'])) {
                if(is_array($currentRow['NMFANTFORN']) && count($currentRow['NMFANTFORN'])) {
                    $selectedApuracao = array();
                    
                    foreach($currentRow['NMFANTFORN'] as $forn) {
                        $row = array_filter($afsApuracao, function($nrApuracao) use ($forn) {
                            return $nrApuracao['CDFORNECED'] === $forn;
                        });
                        $selectedApuracao = array_merge($selectedApuracao, $row);
                    }
                    
                    $afsApuracao = $selectedApuracao;
                }
            }
            
            if(isset($currentRow['NMFILIAL'])) {
                if(is_array($currentRow['NMFILIAL']) && count($currentRow['NMFILIAL'])) {
                    $selectedApuracao = array();
                    
                    foreach($currentRow['NMFILIAL'] as $filial) {
                        $row = array_filter($afsApuracao, function($nrApuracao) use ($filial) {
                            return $nrApuracao['CDFILDEST'] === $filial;
                        });
                        $selectedApuracao = array_merge($selectedApuracao, $row);
                    }
                    
                    $afsApuracao = $selectedApuracao;
                }
            }
            
            if(isset($currentRow['NRAUTOFORN'])) {
                if(is_array($currentRow['NRAUTOFORN']) && count($currentRow['NRAUTOFORN'])) {
                    $selectedApuracao = array();
                    
                    foreach($currentRow['NRAUTOFORN'] as $af) {
                        $row = array_filter($afsApuracao, function($nrApuracao) use ($af) {
                        	return $nrApuracao['NRAUTOFORN'] === $af;
                        });
                        $selectedApuracao = array_merge($selectedApuracao, $row);
                    }
                    
                    $afsApuracao = $selectedApuracao;
                }
            }

            foreach ($afsApuracao as $af) {
                array_push($listaCdforneced, $af['CDFORNECED']);
            }
            
            if(isset($currentRow['ISRESENDEMAIL']) && $currentRow['ISRESENDEMAIL']) {
                if(isset($currentRow['ASSUNTO']) && isset($currentRow['MENSAGEM'])) {
                    if(!empty($currentRow['ASSUNTO']) && !empty($currentRow['MENSAGEM'])) {
                        self::enviarAFporEmail($listaCdforneced, $currentRow['NRAPURPREC'], $currentRow, $currentRow, $currentRow, $afsApuracao, true);   
                    }
                }
            } else {
                self::emitirRelatorioAF($listaCdforneced, $currentRow['NRAPURPREC'], $currentRow, $currentRow, $afsApuracao);
            }
                
            $response->addMessage(new Message($this->logEnvioAF, 'W'));

        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function getAutFornRemanejoEmpresa(DTO\Request $request, DTO\Response $response)
    {
        try {
            $rows = $request->getDataSet()->getRows();
            
            $cdfildest = join("_", array_unique(array_map(function($row) {
                return $row['CDFILDEST'];
            }, $rows)));
            
            $cdfillanc = join("_", array_unique(array_map(function($row) {
                return $row['CDFILLANC'];
            }, $rows)));
            
            $nrAutoForn = join("_", array_unique(array_map(function($row) {
                return $row['NRAUTOFORN'];
            }, $rows)));
            
            $dataSet = $this->sup05100AnaliseAFService->getGridRemanejEmpresa($cdfildest, $cdfillanc, $nrAutoForn);
            
            $response->addDataSet(new DataSet('autoforne', $dataSet));
            
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function getVencRemanejo(DTO\Request\Filter $request, DTO\Response $response) 
	{
		try {
			$filterCriteria = $request->getFilterCriteria();
			
			$cdfildest = self::retornaValorParamFilterCriteria($filterCriteria, 'P_CDFILDEST');
			$cdfillanc = self::retornaValorParamFilterCriteria($filterCriteria, 'P_CDFILLANC');
			$nrautoforn = self::retornaValorParamFilterCriteria($filterCriteria, 'P_NRAUTOFORN');

            $dataSet = $this->contaPagar->retornaVencimentos($cdfildest, $cdfillanc, $nrautoforn);
            
            $response->addDataSet(new DataSet('vencimentos_remanejo', $dataSet));

		} catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
	}
	
	public function getOptionsRemanEmpresa(DTO\Request\Filter $request, DTO\Response $response)
    {
        try {
			$filterCriteria = $request->getFilterCriteria();
			
			$cdfildest = self::retornaValorParamFilterCriteria($filterCriteria, 'P_CDFILIAL');
			$cdfillanc = self::retornaValorParamFilterCriteria($filterCriteria, 'P_CDFILLANC');
			$nrautoforn = self::retornaValorParamFilterCriteria($filterCriteria, 'P_NRAUTOFORN');

            $dataSet = $this->contaPagar->retornaVencimentos($cdfildest, $cdfillanc, $nrautoforn);
            
            $response->addDataSet(new DataSet('options_reman_empresa', $dataSet));

		} catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
        
        /*try {
            $conditions = $request->getFilterCriteria()->getConditions();
            
            $dataSourceName = 'sup05100_orca_fina_empresas';
            $empresas = $this->crudService->getFilteredDataSet($dataSourceName, $conditions)->getRows();
            
            $parents = array();
            $dataSetApura = array();
    
            foreach ($empresas as &$itens) {
                $nrId = $itens['CDEMPRESA'];
                if (!isset($parents[$nrId])) {
                    $parents[$nrId] = array(
                        'ID'        => $nrId,
                        'CDEMPRESA' => $itens['CDEMPRESA'],
                        'NMEMPRESA' => $itens['NMEMPRESA'],
                        'NRORG'     => $itens['NRORG']
                    );
                }
                
                $itens['PARENT'] = $parents[$nrId]['ID'];
                
                $newConditions = array();
                array_push($newConditions, 
                    array('columnName' => 'P_PARENT',       'operator' => '=', 'value' => $itens['PARENT']),
                    array('columnName' => 'P_CDEMPRESA',    'operator' => '=', 'value' => $itens['CDEMPRESA'])
                );
                
                $dataSourceName = 'sup05100_orca_fina_conta_pagar';

                $dataSetApura = array_merge($dataSetApura, $this->crudService->getFilteredDataSet($dataSourceName, $newConditions)->getRows());
            };
            
            $newParents = array_values($parents);
            $child = array_values($dataSetApura);
            $returnDataset = array_merge($newParents, $child);
            
            $response->addDataSet(new DataSet("options_reman_empresa", $returnDataset));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }*/
    }
    
    public function getOrcaFina(DTO\Request\Row $request, DTO\Response $response)
    {
        $rows = $request->getRow();
        foreach($rows as $row) {
            $this->contaPagar->retornaVencimentos($row['CDFILDEST'], $row['CDFILLANC'], $row['NRAUTOFORN']);
        }
       /* function TrdmAnaliseAF.getOrcaFina(OLEDADOS: OleVariant; const P_CDFILDEST,
          P_CDFILLANC, P_NRAUTOFORN, P_CDEMRESA, P_CDTPCONTA,
          P_CDCLASFINA: WideString): OleVariant;
        var
           cdsAux: TClientDataSet;
           oleResult: oleVariant;
        begin
           SetComplete;
           Result := VarArrayCreate([0,1],varVariant);
           try
              try
                 cdsAutFOrca.Close;
                 cdsAutFOrca.CreateDataSet;
                 cdsVencimentos.Close;
                 cdsVencimentos.CreateDataSet;
                 if OLEDADOS <> null then begin
                    cdsAux := TClientDataSet.Create(nil);
                    cdsAux.Data := OLEDADOS;
        
                    cdsAux.First;
                    while not cdsAux.Eof do begin
                       try
                          dtmGeraContaPagar := nil;
                          dtmGeraContaPagar := TdtmGeraContaPagar.Create(Nil);
                          oleResult := dtmGeraContaPagar.ExibeOrcaFinUltrapassado(cdsAux.FieldByName('CDFILDEST').AsString,
                                                                                  cdsAux.FieldByName('CDFILLANC').AsString,
                                                                                  cdsAux.FieldByName('NRAUTOFORN').AsString,
                                                                                  P_CDEMRESA, P_CDTPCONTA, P_CDCLASFINA);
                       finally
                          FreeAndNil(dtmGeraContaPagar);
                       end;
                       cdsAutFOrcaAux.Data := oleResult[0];
                       with cdsAutFOrcaAux do begin
                          First;
                          while not Eof do begin
                             cdsAutFOrca.Insert;
                             cdsAutFOrcaCDFILDEST.AsString   := cdsAutFOrcaAuxCDFILDEST.AsString;
                             cdsAutFOrcaCDFILLANC.AsString   := cdsAutFOrcaAuxCDFILLANC.AsString;
                             cdsAutFOrcaNRAUTOFORN.AsString  := cdsAutFOrcaAuxNRAUTOFORN.AsString;
                             cdsAutFOrcaCDEMPRESA.AsString   := cdsAutFOrcaAuxCDEMPRESA.AsString;
                             cdsAutFOrcaNMEMPRESA.AsString   := cdsAutFOrcaAuxNMEMPRESA.AsString;
                             cdsAutFOrcaCDTPCTPAGAR.AsString := cdsAutFOrcaAuxCDTPCTPAGAR.AsString;
                             cdsAutFOrcaDSTPCTPAGAR.AsString := cdsAutFOrcaAuxDSTPCTPAGAR.AsString;
                             cdsAutFOrcaCDCLASSFINA.AsString := cdsAutFOrcaAuxCDCLASSFINA.AsString;
                             cdsAutFOrcaDSCLASSFINA.AsString := cdsAutFOrcaAuxDSCLASSFINA.AsString;
                             cdsAutFOrca.Post;
                             Next;
                          end;
                       end;
        
                       cdsVencimentosAux.Data := oleResult[1];
                       with cdsVencimentosAux do begin
                          First;
                          while not Eof do begin
                             cdsVencimentos.Insert;
                             cdsVencimentosCDFILDEST.AsString      := cdsVencimentosAuxCDFILDEST.AsString;
                             cdsVencimentosCDFILLANC.AsString      := cdsVencimentosAuxCDFILLANC.AsString;
                             cdsVencimentosNRAUTOFORN.AsString     := cdsVencimentosAuxNRAUTOFORN.AsString;
                             cdsVencimentosDTVENCIMENTO.AsDateTime := cdsVencimentosAuxDTVENCIMENTO.AsDateTime;
                             cdsVencimentosVRVENCIMENTOAF.AsFloat  := cdsVencimentosAuxVRVENCIMENTOAF.AsFloat;
                             cdsVencimentosVRREALORCA.AsFloat      := cdsVencimentosAuxVRREALORCA.AsFloat;
                             cdsVencimentosVRORCAFINA.AsFloat      := cdsVencimentosAuxVRORCAFINA.AsFloat;
                             cdsVencimentosVREXTOLEORCA.AsFloat    := cdsVencimentosAuxVREXTOLEORCA.AsFloat;
                             cdsVencimentosDTORCAFINA.AsString     := cdsVencimentosAuxDTORCAFINA.AsString;
                             cdsVencimentosIDREMANEJA.AsString     := cdsVencimentosAuxIDREMANEJA.AsString;
                             cdsVencimentos.Post;
                             Next;
                          end;
                       end;
                       cdsAux.Next;
                    end;
                 end
                 else begin
                    try
                       dtmGeraContaPagar := nil;
                       dtmGeraContaPagar := TdtmGeraContaPagar.Create(Nil);
                       oleResult := dtmGeraContaPagar.ExibeOrcaFinUltrapassado(P_CDFILDEST, P_CDFILLANC, P_NRAUTOFORN,
                                                                               P_CDEMRESA, P_CDTPCONTA, P_CDCLASFINA);
                    finally
                       FreeAndNil(dtmGeraContaPagar);
                    end;
        
                    cdsVencimentosAux.Data := oleResult[1];
                    with cdsVencimentosAux do begin
                       First;
                       while not Eof do begin
                          if not cdsVencimentos.Locate('CDFILDEST;CDFILLANC;NRAUTOFORN;DTORCAFINA',
                                              VarArrayof([cdsVencimentosAuxCDFILDEST.AsString,
                                                          cdsVencimentosAuxCDFILLANC.AsString,
                                                          cdsVencimentosAuxNRAUTOFORN.AsString,
                                                          cdsVencimentosAuxDTORCAFINA.AsString]), [loCaseInsensitive]) then begin
                             cdsVencimentos.Insert;
                             cdsVencimentosCDFILDEST.AsString      := cdsVencimentosAuxCDFILDEST.AsString;
                             cdsVencimentosCDFILLANC.AsString      := cdsVencimentosAuxCDFILLANC.AsString;
                             cdsVencimentosNRAUTOFORN.AsString     := cdsVencimentosAuxNRAUTOFORN.AsString;
                             cdsVencimentosDTVENCIMENTO.AsDateTime := cdsVencimentosAuxDTVENCIMENTO.AsDateTime;
                             cdsVencimentosVRVENCIMENTOAF.AsFloat  := cdsVencimentosAuxVRVENCIMENTOAF.AsFloat;
                             cdsVencimentosVRREALORCA.AsFloat      := cdsVencimentosAuxVRREALORCA.AsFloat;
                             cdsVencimentosVRORCAFINA.AsFloat      := cdsVencimentosAuxVRORCAFINA.AsFloat;
                             cdsVencimentosVREXTOLEORCA.AsFloat    := cdsVencimentosAuxVREXTOLEORCA.AsFloat;
                             cdsVencimentosDTORCAFINA.AsString     := cdsVencimentosAuxDTORCAFINA.AsString;
                             cdsVencimentosIDREMANEJA.AsString     := cdsVencimentosAuxIDREMANEJA.AsString;
                             cdsVencimentos.Post;
                             Next;
                          end
                          else begin
                             Next;
                          end;
                       end;
                    end;
                 end;
                 Result[0] := cdsAutFOrca.Data;
                 Result[1] := cdsVencimentos.Data;
              except
                 on E:Exception do begin
                    Result := ADOExceptionMessage(E, adcAnaliseAF);
                 end;
              end;
           finally
              FreeAndNil(cdsAux);
           end;
        end;*/
    }
    
    /*
        function TdtmGeraContaPagar.ExibeOrcaFinUltrapassado(stCdFiliDest,
          stCdFiliLanc, stNrAutoForn, stEmpresa, stTpConta, stClassFina: String): OleVariant;
        var
           stCocatena, stEmpAutForn, stClassFinc, stCdTpCtPagar, stCdTipoOper: String;
           dtmApurVenc: TdtmApurVenc;
           dtIni, dtFin: TDateTime;
           inCount: Integer;
        begin
           Result := VarArrayCreate([0,1],varVariant);
           dtmApurVenc := nil;
           cdsAutFOrca.CreateDataSet;
           cdsVencimentos.CreateDataSet;
           try
              dtmApurVenc := TdtmApurVenc.Create(nil);
              dtmApurVenc.CriaApurVenc(adqAutoForne.Connection);
              stCocatena := stCdFiliDest + '||' + stCdFiliLanc + '||' + stNrAutoForn;
              with adqParCmpSup do begin
                 Close;
                 ParamByName('P_CDFILIAL').Value := stCdFiliDest;
                 Open;
                 stCdTipoOper := adqParCmpSup.FieldByName('CDTIPOOPER').AsString;
                 if EmptyString(adqParCmpSup.FieldByName('CDTIPOOPER').AsString) then
                    stCdTipoOper := '01';
              end;
        
              { Busca a Codigo de Classificacao financeira em TIPOCTPAGAR }
              if EmptyString(stTpConta) then begin
                 with adqClassFinanc do begin
                    Close;
                    ParamByName('P_CDFILDEST' ).Value := stCdFiliDest;
                    ParamByName('P_CDFILLANC' ).Value := stCdFiliLanc;
                    ParamByName('P_NRAUTOFORN').Value := stNrAutoForn;
                    Open;
                    stClassFinc   := adqClassFinanc.FieldByName('CDCLASSFINA').AsString;
                    stCdTpCtPagar := adqClassFinanc.FieldByName('CDTPCTPAGAR').AsString;
                    if IsEmpty then begin
                       with adqClassTpOp do begin
                          Close;
                          ParamByName('P_CDTIPOOPER').Value := stCdTipoOper;
                          Open;
                          stClassFinc   := adqClassTpOp.FieldByName('CDCLASSFINA').AsString;
                          stCdTpCtPagar := adqClassTpOp.FieldByName('CDTPCTPAGAR').AsString;
                       end;
                    end;
                 end;
              end
              else begin
                 stCdTpCtPagar := stTpConta;
                 stClassFinc   := stClassFina;
              end;
        
              { Para cada data de entrega de uma AF }
              with adqAutoforne do begin
                 Close;
                 ParamByName('P_CDFILDEST' ).Value := stCdFiliDest;
                 ParamByName('P_CDFILLANC' ).Value := stCdFiliLanc;
                 ParamByName('P_NRAUTOFORN').Value := stNrAutoForn;
                 Open;
              end;
              if EmptyString(stEmpresa) then begin
                 stEmpAutForn := adqAutoforne.FieldByName('CDEMPRESA').AsString;
              end
              else begin
                 stEmpAutForn := stEmpresa;
              end;
              adqAutoforne.First;
              while not adqAutoforne.EOF do begin
                 vaContaPagar := dtmApurVenc.CriaVencimentos('A', stCocatena, adqAutoforne.FieldByName('VRTOTAL_REST').AsFloat,
                                                             adqAutoforne.FieldByName('DTPROGENTAF').AsDateTime, adqAutoforne.FieldByName('DTPROGENTAF').AsDateTime);
                 for inCount := 1 to VarArrayHighBound(vaContaPagar ,2) do begin
                    dtIni := PriDiaMes(vaContaPagar[2,inCount]);
                    dtFin := UltimoDiaMes(vaContaPagar[2,inCount]);
        
                    adqParamGeral.Close;
                    adqParamGeral.Open;
        
                    with adqOrcaFina do begin
                       Close;
                       ParamByName('P_CDEMPRESA').Value   := stEmpAutForn;
                       if adqParamGeral.FieldByName('IDTPORCAFINA').AsString = 'F' then begin
                          ParamByName('P_CDFILIAL').Value := stCdFiliDest;
                       end
                       else begin
                          ParamByName('P_CDFILIAL').Clear;
                       end;
                       ParamByName('P_CDCLASSFINA').Value := stClassFinc;
                       ParamByName('P_DTINI').Value       := dtIni;
                       ParamByName('P_DTFIN').Value       := dtFin;
                       Open;
                    end;
        
                    if not cdsAutFOrca.Locate('CDFILDEST;CDFILLANC;NRAUTOFORN',
                                              VarArrayof([stCdFiliDest, stCdFiliLanc, stNrAutoForn]), [loCaseInsensitive]) then begin
                       cdsAutFOrca.Insert;
                       cdsAutFOrcaCDFILDEST.AsString   := stCdFiliDest;
                       cdsAutFOrcaCDFILLANC.AsString   := stCdFiliLanc;
                       cdsAutFOrcaNRAUTOFORN.AsString  := stNrAutoForn;
                       with adqNmEmpresa do begin
                          Close;
                          ParamByName('P_CDEMPRESA').Value := stEmpAutForn;
                          Open;
                          if not IsEmpty then begin
                             cdsAutFOrcaCDEMPRESA.AsString   := adqNmEmpresa.FieldByName('CDEMPRESA').AsString;
                             cdsAutFOrcaNMEMPRESA.AsString   := adqNmEmpresa.FieldByName('NMEMPRESA').AsString;
                          end;
                       end;
                       with adqDsTpConta do begin
                          Close;
                          ParamByName('P_CDTPCTPAGAR').Value := stCdTpCtPagar;
                          Open;
                          if not IsEmpty then begin
                             cdsAutFOrcaCDTPCTPAGAR.AsString := adqDsTpConta.FieldByName('CDTPCTPAGAR').AsString;
                             cdsAutFOrcaDSTPCTPAGAR.AsString := adqDsTpConta.FieldByName('DSTPCTPAGAR').AsString;
                             cdsAutFOrcaCDCLASSFINA.AsString := adqDsTpConta.FieldByName('CDCLASSFINA').AsString;
                             cdsAutFOrcaDSCLASSFINA.AsString := adqDsTpConta.FieldByName('DSCLASSFINA').AsString;
                          end;
                       end;
                       cdsAutFOrca.Post;
                    end;
        
                    cdsVencimentos.Insert;
                    cdsVencimentosCDFILDEST.AsString     := stCdFiliDest;
                    cdsVencimentosCDFILLANC.AsString     := stCdFiliLanc;
                    cdsVencimentosNRAUTOFORN.AsString    := stNrAutoForn;
                    cdsVencimentosDTVENCIMENTO.AsDateTime:= vaContaPagar[2,inCount];
                    cdsVencimentosVRVENCIMENTOAF.AsFloat := RoundDec(vaContaPagar[3,inCount], 2);
                    cdsVencimentosVRREALORCA.AsFloat     := RoundDec(adqOrcaFina.FieldByName('VRREALORCA').AsFloat, 2);
                    cdsVencimentosVRORCAFINA.AsFloat     := RoundDec(adqOrcaFina.FieldByName('VRORCAFINA').AsFloat, 2);
                    cdsVencimentosVREXTOLEORCA.AsFloat   := RoundDec(adqOrcaFina.FieldByName('VREXTOLEORCA').AsFloat, 2);
                    cdsVencimentosDTORCAFINA.AsString    := FormatDateTime('mm/yyyy', vaContaPagar[2,inCount]);
                    cdsVencimentosIDREMANEJA.AsString    := 'N';
                    cdsVencimentos.Post;
                 end;
                 adqAutoforne.Next;
        
                 if ((not EmptyString(stEmpresa)) and (EmptyString(stTpConta))) then begin
                    with acmUpdAutFOrca do begin
                       ParamByName('P_CDFILDEST'     ).Value := stCdFiliDest;
                       ParamByName('P_CDFILLANC'     ).Value := stCdFiliLanc;
                       ParamByName('P_NRAUTOFORN'    ).Value := stNrAutoForn;
                       Execute;
                    end;
                 end;
              end;
              Result[0] := cdsAutFOrca.Data;
              Result[1] := cdsVencimentos.Data;
           finally
              dtmApurVenc.Free;
           end;
        end;
    */
    
}